/*                                                       
HTML.cc
*/

#include "HTML.h"
extern Language *L;
extern CCLIENT *thecclient;

HTML::HTML (FILE *FR)
  {
  MAXMSGSFORPAGE = DEFOPT_MSGSPERINDEXPAGE;
  fout = FR;
  iout = -1;
  OneSocket = NULL;
  XBuffer = "";
  }  

HTML::HTML (int FR)
  {
  MAXMSGSFORPAGE = DEFOPT_MSGSPERINDEXPAGE;
  fout = NULL;
  iout = FR;
  OneSocket = NULL;
  XBuffer = "";  
  }
   
HTML::HTML (XSocket *aXS)
  {
  MAXMSGSFORPAGE = DEFOPT_MSGSPERINDEXPAGE;
  fout = NULL;
  iout = -1;
  OneSocket = aXS;
  XBuffer = "";
  }
 
HTML::~HTML()
  {
  }  
  
void HTML::setMAXMSGSFORPAGE (long maxmsgs)
  {
  MAXMSGSFORPAGE = maxmsgs;
  }          

void HTML::WRITE (const char *fmt, ...)
  {
  TBuffer linea;
  va_list args;
  va_start (args, fmt);
  xvsnprintf (linea, CMAXBUFFER, fmt, args);  
  if (iout != -1) write (iout, linea, strlen(linea));
  else if (OneSocket != NULL) 
    {    
    OneSocket->Puts (linea);
    }
  else {fprintf (fout, linea); fflush (fout);}
  va_end (args);      
  }  

void HTML::pcWRITE (const char *pc)
  {
  if (iout != -1) write (iout, (char *)pc, strlen(pc));
  else if (OneSocket != NULL) 
    {
    OneSocket->Puts (pc);
    }
  else {fprintf (fout, pc); fflush (fout);}
  }  
  
void HTML::pcWRITEBuffer (const char *pc, xulong len)
  {
  if (pc == NULL) {LOG("Null in pcWRITEBuffer"); return;}
  if (iout != -1) write (iout, (char *)pc, (uint)len);
  else if (OneSocket != NULL) 
    {
    OneSocket->Write (pc, (uint)len);    
    }
  else {write (fileno(fout), (char *)pc, (uint)len); fflush (fout);}
  }    
  
void HTML::printCookie (const char *cookieline)
  {
  if ((cookieline != NULL) && (strcmp(cookieline, "") != 0)) 
    {
    WRITE ("Set-Cookie: %s\n", cookieline);
    }
  }   
  
void HTML::Head (const char *contenttype, const char *name, const char *filename)
  {
  if ((name     != NULL) && (strcmp(name, "")     != 0)) {WRITE ("Name: %s\n", name);}
  if ((filename != NULL) && (strcmp(filename, "") != 0)) {WRITE ("Filename: %s\n", filename);}
  WRITE ("Content-type: %s\n", contenttype);  
  WRITE ("\n");
  } 
  
void HTML::HeadInline (const char *contenttype, const char *filename, xulong len)
  {        
  if ((strcmp (filename, "") != 0) && (filename != NULL) && (len > 0))
    {
    WRITE ("Content-Disposition: inline; filename=%s\n", filename);
    WRITE ("Content-Length: %ld\n", len);
    }
  WRITE ("Content-type: %s\n", contenttype);  
  WRITE ("\n");
  }   

void HTML::EmulatePageNotFound (void)
  {
  //WRITE ("HTTP/1.0 404 Not Found\n");
  WRITE ("Connection: close\n");
  WRITE ("Status: 404 Not Found\n");
  WRITE ("Content-type: text/html\n");
  WRITE ("\n");
  WRITE ("%s\n", HTML_DOCTYPE);
  WRITE ("<HTML><HEAD>\n");
  WRITE ("<TITLE>404 Not Found</TITLE>\n");
  WRITE ("</HEAD><BODY>\n"); 
  WRITE ("<H1>Not Found</H1>\n");
  WRITE ("The requested URL was not found on this server.<P>\n");
  WRITE ("</BODY></HTML>\n");  
  }
  
void HTML::HTMLBegin (void)
  {
  WRITE ("%s\n<HTML>\n", HTML_DOCTYPE);
  }  
  
void HTML::Title (const char *title, const char *bgcolor, const char *txtcolor, const char *img_bg, bool include_js, int refresh_time, const char *newurl)
  {
  WRITE ("<HEAD>\n");
  WRITE ("<TITLE>%s</TITLE>\n", title);
  if (include_js == true)
    {
    //Here we can put JavaScripts
    }
       
  if ((refresh_time > 0) && (newurl != NULL))
    {
    //Time arrives in minutes and transform to seconds
    //DEBUG WRITE ("<META HTTP-EQUIV=\"refresh\" CONTENT=\"%d;URL=%s\">\n", refresh_time, newurl);  
    WRITE ("<META HTTP-EQUIV=\"refresh\" CONTENT=\"%d;URL=%s\">\n", refresh_time * 60, newurl);
    }
  
  WRITE ("</HEAD>\n");
  if ((strcmp (img_bg, " ") != 0) && (img_bg != NULL))
    {
    WRITE ("<BODY background=\"%s\" text=\"%s\" bgcolor=\"%s\" link=\"#3333CC\" vlink=\"#3333CC\" alink=\"#3333CC\">\n", 
           img_bg, txtcolor, bgcolor);            
    }
  else
    {
    WRITE ("<BODY text=\"%s\" bgcolor=\"%s\" link=\"#3333CC\" vlink=\"#3333CC\" alink=\"#3333CC\">\n", 
            txtcolor, bgcolor);            
    }
  }  
  
void HTML::HTMLEnd (const char *bgtablecopyrightcolor)
  {
  Linea ();  
  BeginTable (0, 2, 1, 100);
  if (bgtablecopyrightcolor == NULL) {WRITE ("<TR bgcolor=\"%s\">", DEFCOLOR_BGTABLECOPYRIGHT);}
  else {WRITE ("<TR bgcolor=\"%s\">", bgtablecopyrightcolor);}
  BeginTDColSpanCenter (7); Small (L->get (L_COPYRIGHT)); EndTD();
  EndTR();
  EndTable();
  if (SHOW_VALIDATE_HTML_URL == 1) {WRITE ("<BR><A HREF=\"http://www.htmlhelp.com/cgi-bin/validate.cgi?url=referer&amp;input=yes\">Validate HTML code</A>\n");}
  WRITE ("</BODY>\n");
  WRITE ("</HTML>\n"); 
  }                  
  
void HTML::BR (void)
  {
  WRITE ("<BR>\n");
  }                    

void HTML::JavaScript (const char *codigo)
  {
  WRITE ("\n<SCRIPT LANGUAGE=\"JavaScript\" TYPE=\"text/javascript\">\n");
  WRITE ("<!--\n");
  WRITE ("%s", codigo);
  WRITE ("\n// -->\n");
  WRITE ("</SCRIPT>\n");
  }

void HTML::Small (const char *linea)
  {
  WRITE ("<SMALL>%s</SMALL>\n", linea);
  }

void HTML::Rectangle (const char *linea)
  {
  WRITE ("<TABLE BORDER=1><TR><TD>\n", linea);
  WRITE ("%s\n", linea);
  WRITE ("</TD></TR></TABLE>\n");
  }

void HTML::RedRectangle (const char *linea)
  {
  WRITE ("<TABLE BORDER=1><TR><TD>\n", linea);
  WRITE ("<FONT color=\"#FF0000\"><B>%s</B></FONT>\n", linea);
  WRITE ("</TD></TR></TABLE>\n");
  }      

void HTML::H1 (const char *linea)
  {
  WRITE ("<H1>%s</H1>\n", linea);
  }
  
void HTML::H2 (const char *linea)
  {
  WRITE ("<H2>%s</H2>\n", linea);
  }

void HTML::H3 (const char *linea)
  {
  WRITE ("<H3>%s</H3>\n", linea);
  }  
  
void HTML::H4 (const char *linea)
  {
  WRITE ("<H4>%s</H4>\n", linea);
  }

void HTML::H5 (const char *linea)
  {
  WRITE ("<H5>%s</H5>\n", linea);
  }  
  
void HTML::BeginTable (int border, int cellpadding, int cellspacing, int widthpercent)
  {
  WRITE ("<TABLE cellpadding=\"%d\" cellspacing=\"%d\" width=\"%d%%\" BORDER=\"%d\">\n", cellpadding, cellspacing, widthpercent, border);
  }

void HTML::EndTable (void)
  {
  WRITE ("</TABLE>\n");
  }

void HTML::BeginTR (void)
  {
  WRITE ("<TR>\n");
  }

void HTML::BeginTR (const char *bgcolor)
  {
  WRITE ("<TR bgcolor=\"%s\">\n", bgcolor);
  }

void HTML::EndTR (void)
  {
  WRITE ("</TR>\n");
  }

void HTML::BeginTD (void)
  {
  WRITE ("<TD>\n");
  WRITE ("<FONT size=2 face=\"%s\" color=\"%s\">\n", TABLEFACEFONT, BLACK);
  }
  
void HTML::BeginTDColSpan (int acolspan)
  {
  WRITE ("<TD colspan=\"%d\">\n", acolspan);
  WRITE ("<FONT size=2 face=\"%s\" color=\"%s\">\n", TABLEFACEFONT, BLACK);
  }
  
void HTML::BeginTDColSpanCenter (int acolspan)
  {
  WRITE ("<TD align=\"center\" colspan=\"%d\">\n", acolspan);
  WRITE ("<FONT size=2 face=\"%s\" color=\"%s\">\n", TABLEFACEFONT, BLACK);
  }  
  
void HTML::BeginTDCenter (void)
  {
  WRITE ("<TD align=\"center\">\n");
  WRITE ("<FONT size=2 face=\"%s\" color=\"%s\">\n", TABLEFACEFONT, BLACK);
  }
  
void HTML::BeginTD (const char *bgcolor)
  {
  WRITE ("<TD bgcolor=\"%s\">\n", bgcolor);
  WRITE ("<FONT size=2 face=\"%s\" color=\"%s\">\n", TABLEFACEFONT, BLACK);
  }      

void HTML::BeginTD (const char *bgcolor, const char *align)
  {
  WRITE ("<TD bgcolor=\"%s\" align=\"%s\">\n", bgcolor, align);
  WRITE ("<FONT size=2 face=\"%s\" color=\"%s\">\n", TABLEFACEFONT, BLACK);
  }

void HTML::BeginTD (const char *bgcolor, const char *align, int colspan)
  {
  WRITE ("<TD bgcolor=\"%s\" align=\"%s\" colspan=\"%d\">\n", bgcolor, align, colspan);
  WRITE ("<FONT size=2 face=\"%s\" color=\"%s\">\n", TABLEFACEFONT, BLACK);
  }

void HTML::BeginTD (int widthpercent)
  {
  WRITE ("<TD width=\"%d%%\">\n", widthpercent);    
  WRITE ("<FONT size=\"2\" face=\"%s\" color=\"%s\">\n", TABLEFACEFONT, BLACK);
  }
  
void HTML::BeginTD (bool nowrap, int widthpercent, int rowspan, const char *facefont, int sizefont, const char *align)
  {
  if (nowrap == true)
    {
    WRITE ("<TD nowrap width=\"%d%%\" rowspan=\"%d\" align=\"%s\">\n", widthpercent, rowspan, align);
    }
  else
    {
    WRITE ("<TD width=\"%d%%\" rowspan=\"%d\" align=\"%s\">\n", widthpercent, rowspan, align);    
    }
  WRITE ("<FONT size=\"%d\" face=\"%s\" color=\"%s\">\n", sizefont, facefont, BLACK);
  }

void HTML::BeginTD (int widthpercent, const char *facefont, int sizefont, const char *align)
  {
  WRITE ("<TD width=\"%d%%\" align=\"%s\">\n", widthpercent, align);      
  WRITE ("<FONT size=\"%d\" face=\"%s\" color=\"%s\">\n", sizefont, facefont, BLACK);
  }

void HTML::EndTD (void)
  {
  WRITE ("</FONT></TD>\n");
  }

void HTML::EmptyCellx1 (void)
  {
  BeginTD (); WRITE(HTML_HSP); EndTD();
  }
  
void HTML::EmptyCellx2 (void)
  {
  BeginTD (); WRITE(HTML_HSP); EndTD();
  BeginTD (); WRITE(HTML_HSP); EndTD();
  }                

void HTML::BeginBold (void)
  {
  WRITE ("<B>\n");
  }

void HTML::EndBold (void)
  {
  WRITE ("</B>\n");
  }

void HTML::BeginItalic (void)
  {
  WRITE ("<I>\n");
  }

void HTML::EndItalic (void)
  {
  WRITE ("</I>\n");
  }
void HTML::BeginUnderline (void)
  {
  WRITE ("<U>\n");
  }

void HTML::EndUnderline (void)
  {
  WRITE ("</U>\n");
  }

void HTML::BeginFont (const char *face, int size, const char *color)
  {
  WRITE ("<FONT size=\"%d\" face=\"%s\" color=\"%s\">\n", size, face, color);
  }
  
void HTML::EndFont (void)
  {
  WRITE ("</FONT>\n");
  }        

void HTML::Label (const char *linea)
  {
  WRITE ("<B>%s</B>\n", linea);
  }  

void HTML::RedLabel (const char *linea)
  {
  WRITE ("<FONT color=\"#FF0000\"><B>%s</B></FONT>\n", linea);
  }

void HTML::GreenLabel (const char *linea)
  {
  WRITE ("<FONT color=%s><B>%s</B></FONT>\n", GREEN, linea);
  }

void HTML::Image (const char *urlimage, const char *label)
  {
  WRITE ("<IMG BORDER=0 SRC=\"%s\" ALT=\"%s\">\n", urlimage, label);
  }
  
void HTML::URL (const char *caption, const char *url)
  {
  WRITE ("<A HREF=\"%s\">%s</A>\n", url, caption);
  }  
                            
void HTML::URLIMAGE (const char *label, const char *url, const char *urlimage)
  {
  WRITE ("<A HREF=\"%s\"><IMG BORDER=0 SRC=\"%s\" ALT=\"%s\"></A>\n", url, urlimage, label);
  }    

void HTML::BeginPre ()
  {
  WRITE ("<PRE>\n");
  }
  
void HTML::EndPre ()
  {
  WRITE ("</PRE>\n");
  }
   
void HTML::BeginCenter ()
  {
  WRITE ("<CENTER>\n");
  }
 
void HTML::EndCenter ()
  {
  WRITE ("</CENTER>\n");
  } 

void HTML::PrintString (const char *linea)
  {
  WRITE ("%s<BR>\n", linea);
  }
  
void HTML::pr (const char *linea)
  {
  WRITE ("%s\n", linea);
  }
        
void HTML::Redirect (const char *newurl)
  {
  WRITE ("Status: 302 Moved\n");
  WRITE ("Location: %s\n", newurl);
  WRITE ("Content-Type: text/html\n\n");
  }  
  
void HTML::BeginNOBR (void)
  {
  WRITE ("<NOBR>\n");
  }
  
void HTML::EndNOBR(void)
  {
  WRITE ("</NOBR>\n");
  }        
  
void HTML::Linea ()
  {
  WRITE ("<HR noshade size=2>\n");
  }
  
void HTML::Error (const char *error)
  {
  WRITE ("<p><h1><FONT color=\"#FF0000\">Error:</font> %s</h1><p>\n", error);
  } 

void HTML::Msg (const char *msg)
  {
  WRITE ("<p><h1>%s</h1><p>\n", msg);
  } 
 
void HTML::ErrorPage (const char *error)
  {                    
  Head ("text/html", "", "");  
  HTMLBegin ();  
  Title (error, WHITE, BLACK, DEFIMG_BG, false, 0, NULL);
  WRITE ("<p><h1><FONT color=\"%s\">Error:</font> %s</h1><p>\n", RED, error);
  HTMLEnd(NULL);      
  } 

void HTML::ErrorPageBack (const char *stlang, const char *user, const char *connid, 
                          UserOptions *uo, const char *error, int way)
  {                    
  Head ("text/html", "", "");  
  HTMLBegin ();  
  Title (error, uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"), uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL);
  switch (way)
    {       
    case 1:
      {
      BeginTable (0, 0, 0, 100);
      BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_MAIN_SIZE"), "center");
      WRITE ("<A HREF=\"%s/mb_index/noop/%s/%s/%s/%d/%d/%d/%ld/\">", PATH_POSTMAN, stlang, user, connid, uo->getValidator(), 
             uo->getPageId(), CMD_MB_INDEX, uo->getCurrentPosition());
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_INDEXMAILBOX), L->get(L_INDEXMAILBOX2));
      WRITE (" %s\n", L->get(ALT_BACK));
      WRITE ("</A>");
      EndTD();        
      EmptyCellx2();
      EndTR();
      EndTable (); 
      break;
      }
    case 2:
      {
      BeginTable (0, 0, 0, 100);
      BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_MAIN_SIZE"), "center");
      WRITE ("<A HREF=\"%s/cm_forward/noop/%s/%s/%s/%d/%d/%d/%ld/\">", PATH_POSTMAN, stlang, user, connid, uo->getValidator(), 
             uo->getPageId(), CMD_MB_INDEX, uo->getCurrentPosition());
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_BACK), L->get(L_BACK));
      WRITE (" %s\n", L->get(ALT_BACK));
      WRITE ("</A>");
      EndTD();      
      EmptyCellx2();  
      EndTR();
      EndTable (); 
      break;
      }      
    }
  WRITE ("<p><h1><FONT color=\"%s\">Error:</font> %s</h1><p>\n", RED, error);
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  }                                                       

void HTML::MsgPage (const char *msg, const char *lang)
  {                       
  Head ("text/html", "", "");  
  HTMLBegin ();  
  Title (msg, WHITE, BLACK, DEFIMG_BG, false, 0, NULL);
  WRITE ("<p><h1><FONT color=%s>%s</font></h1><p>\n", GREEN, msg);
  WRITE ("<p><A HREF=\"%s?lang=%s\">%s</A>", PATH_POSTMAN, lang, L->get(MSG_CANRECONNECTHERE));
  HTMLEnd(NULL); 
  } 

void HTML::OkPage (const char *stlang, const char *user, const char *connid, UserOptions *uo, const char *okmsg)
  {
  Head ("text/html", "", "");
  HTMLBegin ();
  Title (okmsg, uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"), uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL);
  WRITE ("<BR><FONT color=%s size=5><B>%s</B></FONT><BR><BR>\n", GREEN, okmsg);
  WRITE ("<A HREF=\"%s/mb_index/noop/%s/%s/%s/%d/%d/%d/%ld/\">", PATH_POSTMAN, stlang, user, connid, uo->getValidator(), 
         uo->getPageId(), CMD_MB_INDEX, uo->getCurrentPosition());
  WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_INDEXMAILBOX), L->get(L_INDEXMAILBOX2));
  Label (L->get(L_PRESS_TO_INDEX));
  WRITE ("</A>");
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  }  
  
void HTML::Ok (const char *linea)
  {
  WRITE ("<p><h1><FONT color=%s>%s</font></h1><p>\n", GREEN, linea);
  } 
            
void HTML::BeginFormPost (const char *name, const char *action)
  {
  WRITE ("<FORM NAME=\"%s\" ACTION=\"%s\" METHOD=\"POST\">\n", name, action);
  } 
  
void HTML::BeginFormPostMultipartFormData (const char *name, const char *action)
  {
  WRITE ("<FORM NAME=\"%s\" ACTION=\"%s\" METHOD=\"POST\" ENCTYPE=\"multipart/form-data\">\n", name, action);
  }                          
                             
void HTML::BeginFormGet (const char *name, const char *action)
  {
  WRITE ("<FORM NAME=\"%s\" ACTION=\"%s\" METHOD=\"GET\">\n", name, action);
  }   
  
void HTML::EndForm ()
  {
  WRITE ("</FORM>\n");
  }   
    
void HTML::FormBeginSelect (const char *formname, const char *selectname, const char *submitbutton, bool use_javascript)
  {
  if ((submitbutton == NULL) || (use_javascript == false))
    {
    WRITE ("<SELECT SIZE=1 NAME=\"%s\">\n", selectname);
    }
  else
    {
    WRITE ("<SELECT SIZE=1 NAME=\"%s\" onChange=\"%s.%s.click()\">\n", selectname, formname, submitbutton);
    }
  }                                  
  
void HTML::FormBeginMultipleSelect (const char *formname, const char *selectname, const char *submitbutton, bool use_javascript)
  {
  if ((submitbutton == NULL) || (use_javascript == false))
    {
    WRITE ("<SELECT MULTIPLE SIZE=10 NAME=\"%s\">\n", selectname);
    }
  else
    {      
    WRITE ("<SELECT MULTIPLE SIZE=10 NAME=\"%s\" onChange=\"%s.%s.click()\">\n", selectname, formname, submitbutton);
    }
  } 

void HTML::FormAddOptionSelect (bool selected, const char *value, const char *label)
  {
  if (selected == true) 
    {
    if (strcmp (value, FORM_SELECT_SEPARATOR) == 0)
      {WRITE ("<OPTION selected DISABLED VALUE=\"%s\">%s\n", FORM_SELECT_SEPARATOR, FORM_SELECT_SEPARATOR);}
    else               
      {WRITE ("<OPTION selected VALUE=\"%s\">%s\n", value, label);}
    }
  else
    {
    if (strcmp (value, FORM_SELECT_SEPARATOR) == 0)
      {WRITE ("<OPTION DISABLED VALUE=\"%s\">%s\n", FORM_SELECT_SEPARATOR, FORM_SELECT_SEPARATOR);}
    else
      {WRITE ("<OPTION VALUE=\"%s\">%s\n", value, label);}
    }
  }

void HTML::FormEndSelect ()
  {
  WRITE ("</SELECT>\n");
  }                                       

void HTML::FormHiddenQuoted (const char *name, const char *value)
  {
  WRITE ("<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s%s%s\">\n", name, HTML_DCOMILLAS, value, HTML_DCOMILLAS);
  }
   
void HTML::FormHidden (const char *name, const char *value)
  {
  WRITE ("<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n", name, value);
  } 

void HTML::FormHidden (const char *name, int value)
  {
  WRITE ("<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%d\">\n", name, value);
  } 

void HTML::FormEdit (int max, int size, const char *name, const char *value)
  {
  WRITE ("<INPUT TYPE=TEXT NAME=\"%s\" MAXLENGTH=\"%d\" SIZE=\"%d\" VALUE=\"%s\">\n", 
           name, max, size, value ? value : "");
  //If value is not NULL, print value, else print ""
  }  

void HTML::FormFile (int max, int size, const char *name, const char *value, const char *accept)
  {
  WRITE ("<INPUT TYPE=FILE NAME=\"%s\" MAXLENGTH=\"%d\" SIZE=\"%d\" VALUE=\"%s\" ACCEPT=\"%s\">\n", 
           name, max, size, value, accept);
  }

void HTML::FormTextArea (int rows, int cols, const char *name, const char *value, bool readonly)
  {
  if (readonly == true)
    {
    WRITE ("<TEXTAREA READONLY NAME=\"%s\" ROWS=\"%d\" COLS=\"%d\" WRAP=\"HARD\">\n", name, rows, cols);
    }
  else
    {
    WRITE ("<TEXTAREA NAME=\"%s\" ROWS=\"%d\" COLS=\"%d\" WRAP=\"HARD\">\n", name, rows, cols);
    }  
  pcWRITEBuffer (value, strlen (value));
  WRITE ("</TEXTAREA>\n");
  }  
   
void HTML::FormPassword (int max, int size, const char *name, const char *value)
  {
  WRITE ("<INPUT TYPE=PASSWORD NAME=\"%s\" MAXLENGTH=\"%d\" SIZE=\"%d\" VALUE=\"%s\">\n", 
           name, max, size, value);
  }  
  
void HTML::FormCheckbox (const char *name, const char *value, bool checked)
  {
  if (checked == true)
    WRITE ("<INPUT TYPE=\"CHECKBOX\" NAME=\"%s\" VALUE=\"%s\" CHECKED>\n", name, value);
  else
    WRITE ("<INPUT TYPE=\"CHECKBOX\" NAME=\"%s\" VALUE=\"%s\">\n", name, value);    
  }      
  
void HTML::JSFormCheckbox (const char *name, const char *value, const char *onclick, bool checked)
  {
  if (checked == true)
    WRITE ("<INPUT TYPE=\"CHECKBOX\" NAME=\"%s\" onClick=\"%s\" VALUE=\"%s\" CHECKED>\n", name, onclick, value);
  else
    WRITE ("<INPUT TYPE=\"CHECKBOX\" NAME=\"%s\" onClick=\"%s\" VALUE=\"%s\">\n", name, onclick, value);    
  }    
  
void HTML::FormImage (const char *name, const char *src)
  {
  WRITE ("<INPUT TYPE=IMAGE NAME=\"%s\" SRC=\"%s\">\n", name, src);
  }  
  
void HTML::FormButton (const char *name, const char *value)
  {
  WRITE ("<INPUT TYPE=SUBMIT NAME=\"%s\" VALUE=\"%s\">\n", name, value);
  }  
        
void HTML::FormReset (const char *value)
  {
  WRITE ("<INPUT TYPE=RESET VALUE=\"%s\">\n", value);
  }
  
void HTML::soErrorPage (const char *error)
  {                    
  fprintf (stdout, "Content-type: text/html\n\n");     
  fprintf (stdout, "<p><h1><FONT color=\"#FF0000\">Error:</font> %s</h1><p>\n", error);
  }  

void HTML::dumpSL (StringList *SL, bool PutNewLines)
  {
  Head ("text/plain", "", "");
  for (int i=0; i< SL->Count(); ++i)
    {
    if (PutNewLines == false)
      {
      WRITE (SL->getString(i).cstr());
      }
    else
      {
      WRITE ("%s\n", SL->getString(i).cstr());
      }
    }
  }

void HTML::dumpPChar (const char *pc)
  {
  Head ("text/plain", "", "");
  if (pc == NULL) pcWRITE ("NULL"); 
  else pcWRITE (pc);
  }                  
  
void HTML::LineaLanguage ()
  {
  TBuffer Buf;
  Linea ();
  PrintString (L->get(L_CHOOSELANGUAGE));
  sprintf (Buf, "%s?lang=eng", PATH_POSTMAN); URL ("English", Buf);
  sprintf (Buf, "%s?lang=spa", PATH_POSTMAN); URL ("Espa&ntilde;ol", Buf);
  sprintf (Buf, "%s?lang=val", PATH_POSTMAN); URL ("Valenci&agrave;", Buf);
  sprintf (Buf, "%s?lang=eus", PATH_POSTMAN); URL ("Euskera", Buf);  
  }
                  
void HTML::PrintStatusLine (const char *user, UserOptions *uo, bool displayClock)
  {
  WRITE ("<TABLE width=\"100%%\">\n<TR>\n");
  if (FileExists (FILE_FOR_NO_ACCEPT_NEW_CONNS) == true)
    {
    TBuffer amsg;
    GetBufferFromFile (FILE_FOR_NO_ACCEPT_NEW_CONNS, amsg, CMAXBUFFER);
    WRITE ("<TD align=left><B><FONT color=\"#FF0000\"><BLINK>%s</BLINK></FONT></B></TD>\n<TD align=right><B><FONT size=2 color=\"%s\">%s: %s@%s (%s)</FONT></B></TD>\n",
              amsg, uo->getSkinAtribProp("COLOR_FONT_STATUSLINE"), 
              L->get(L_USER), user, uo->getImapServer(), 
              thecclient->getMailboxName());
    }
  else
    {
    if (thecclient->getError () == false)
      {
      WRITE ("<TD align=left><B>%s</B></TD>\n<TD align=right><B><FONT color=\"%s\">%s: %s@%s (%s)</FONT></B></TD>\n",
             uo->getStatusLine(), uo->getSkinAtribProp("COLOR_FONT_STATUSLINE"), 
             L->get(L_USER), user, uo->getImapServer(), 
             thecclient->getMailboxName());
      }
    else
      {
      WRITE ("<TD align=left><B>%s</B></TD>\n<TD align=right><B><FONT color=\"%s\">%s: %s@%s (%s)</FONT></B></TD>\n",
             thecclient->getErrorString(), uo->getSkinAtribProp("COLOR_FONT_STATUSLINE"), 
             L->get(L_USER), user, uo->getImapServer(), 
             thecclient->getMailboxName());
      }  
    }
  
  //Only is seen in IE4+/N6+
  if ((displayClock == true) && (uo->get_use_javascript() == true) && (uo->getShowClock() == true))
    {        
    WRITE ("<TD width=\"1%%\" align=right>");  
    JavaScript (DEF_JavaScriptIDClock);    
    JavaScript (DEF_JavaScriptClock);
    WRITE ("</TD>"); 
    }

  WRITE ("\n</TR></TABLE>\n");
  }
  
void HTML::BotoneraDisplayMsg (const char *lang, const char *user, const char *connid, UserOptions *uo, bool isdeleted, 
                               ULongList *LL, bool showreplyall, bool isTopBar)
  {
  long actualpos, totalmsgs, trueactualpos, truenummsg;
  int validator, pageid, actualcmd;
  
  actualpos = uo->getCurrentPosition();
  totalmsgs = uo->getTotalMsgs();
  validator = uo->getValidator();
  pageid    = uo->getPageId();
  actualcmd = CMD_MG_DISPLAY;
  trueactualpos = LL->indexOf (actualpos);
  
  if (isTopBar == true) 
    {
    BeginTable (0, 0, 0, 100);
    BeginTR(uo->getSkinAtribProp("COLOR_BGBOTONERA"));

    //BUTTON BACK
    BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"),  uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
    WRITE ("%s", uo->getLastStringCmd(CMD_MG_DISPLAY, lang, connid, true));
    WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_BACK), L->get(ALT_BACK));
    WRITE (" %s\n", L->get(ALT_BACK));
    WRITE ("</A>");
    EndTD(); 
    BeginTD (10, uo->getSkinAtribProp("FONT_MAIN"),  uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
    //BUTTON HELP   
    BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"),  uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
    WRITE ("<A HREF=\"%s/help/mg_display/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd);
    WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_HELP), L->get(ALT_HELP));
    WRITE (" %s\n", L->get(ALT_HELP));
    WRITE ("</A>");
    EndTD();
    BeginTD (10, uo->getSkinAtribProp("FONT_MAIN"),  uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();         
          
    //BUTTON PREV MSG
    if (trueactualpos > 0)
      {
      truenummsg = LL->elementAt((int)(trueactualpos - 1));
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("<A HREF=\"%s/mg_display/noop/%s/%s/%s/%d/%d/%d/%ld/\">",PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd, truenummsg);
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_PREVMSG), L->get(ALT_PREVMSG));
      WRITE (" %s\n", L->get(ALT_PREVMSG));
      WRITE ("</A>");
      EndTD();
      }
    else
      {
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_PREVMSG_INA), L->get(ALT_PREVMSG_INA));
      WRITE (" %s\n", L->get(ALT_PREVMSG_INA));
      EndTD();
      }
    BeginTD (10, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
    //BUTTON NEXT MSG
    if (trueactualpos < totalmsgs - 1)
      {
      truenummsg = LL->elementAt((int)(trueactualpos + 1));
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("<A HREF=\"%s/mg_display/noop/%s/%s/%s/%d/%d/%d/%ld/\">",PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd, truenummsg);
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_NEXTMSG), L->get(ALT_NEXTMSG));
      WRITE (" %s\n", L->get(ALT_NEXTMSG));
      WRITE ("</A>");
      EndTD();
      }
    else
      {
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_NEXTMSG_INA), L->get(L_NEXTMSG_INA));
      WRITE (" %s\n", L->get(ALT_NEXTMSG_INA));
      EndTD();
      }
    BeginTD (10, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
    //BUTTON REPLY
    BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
    WRITE ("<A HREF=\"%s/cm_reply/noop/%s/%s/%s/%d/%d/%d/%ld/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd, actualpos);
    WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_REPLYMSG), L->get(ALT_REPLYMSG));
    WRITE (" %s\n", L->get(ALT_REPLYMSG));
    WRITE ("</A>");                                                      
    EndTD();   
    if (showreplyall == true)
      {
      /**/BeginTD (10, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON REPLYALL
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("<A HREF=\"%s/cm_replyall/noop/%s/%s/%s/%d/%d/%d/%ld/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd, actualpos);
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_REPLYALLMSG), L->get(ALT_REPLYALLMSG));
      WRITE (" %s\n", L->get(ALT_REPLYALLMSG));
      WRITE ("</A>");                                                      
      EndTD();              
      }      
    if (thecclient->getServerProtocol() == PROTOCOL_NNTP) 
      {      
      /**/BeginTD (10, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON REPLYGROUP
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("<A HREF=\"%s/cm_replygroup/noop/%s/%s/%s/%d/%d/%d/%ld/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd, actualpos);
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_SENDTONNTPGROUP), L->get(ALT_SENDTONNTPGROUP));
      WRITE (" %s\n", L->get(ALT_SENDTONNTPGROUP));
      WRITE ("</A>");                                                      
      EndTD();                    
      }      
    BeginTD (10, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
    //BUTTON FORWARD
    BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
    WRITE ("<A HREF=\"%s/cm_forward/noop/%s/%s/%s/%d/%d/%d/%ld/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd, actualpos);
    WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_FORWARDMSG), L->get(ALT_FORWARDMSG));
    WRITE (" %s\n", L->get(ALT_FORWARDMSG));
    WRITE ("</A>");
    EndTD();              
    BeginTD (10, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
    //BUTTON SAVE ADDRESS
    BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
    WRITE ("<A HREF=\"%s/ab/ab_takeaddress/%s/%s/%s/%d/%d/%d/%ld/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd, actualpos);
    WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_SAVEADDRESS), L->get(ALT_SAVEADDRESS));
    WRITE (" %s\n", L->get(ALT_SAVEADDRESS));
    WRITE ("</A>");
    EndTD();              
    BeginTD (10, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
    if (thecclient->getServerProtocol() == PROTOCOL_IMAP) 
      {      
      //BUTTON DELNEXTMSG
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      if (trueactualpos < totalmsgs - 1) {truenummsg = LL->elementAt((int)(trueactualpos + 1));} else {truenummsg = LL->elementAt((int)(trueactualpos));}
      if (isdeleted == false)
        {
        WRITE ("<A HREF=\"%s/mg_deldisplay/noop/%s/%s/%s/%d/%d/%d/%ld/\">",PATH_POSTMAN, lang, user, connid, validator, pageid, 
               actualcmd, truenummsg);
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_DELNEXTMSG), L->get(ALT_DELNEXTMSG));
        WRITE (" %s\n", L->get(ALT_DELNEXTMSG));
        WRITE ("</A>");                  
        }
      else
        {
        WRITE ("<A HREF=\"%s/mg_undeldisplay/noop/%s/%s/%s/%d/%d/%d/%ld/\">",PATH_POSTMAN, lang, user, connid, validator, pageid, 
               actualcmd, truenummsg);
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_UNDELNEXTMSG), L->get(ALT_UNDELNEXTMSG));
        WRITE (" %s\n", L->get(ALT_UNDELNEXTMSG));
        WRITE ("</A>");                          
        }
      EndTD();
      }
    BeginTD (10, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
    //BUTTON LOGOUT   
    BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
    if (uo->getConfirmLogout() == true)    
      {
      WRITE ("<A HREF=\"%s/conf_act/logout/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd);
      }
    else
      {  
      WRITE ("<A HREF=\"%s/logout/noop/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd);
      }
    WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_LOGOUT), L->get(ALT_LOGOUT));
    WRITE (" %s\n", L->get(ALT_LOGOUT));
    WRITE ("</A>");
    EndTD();
    EndTR();
    EndTable (); 
    PrintStatusLine (user, uo, true);
    }
  else //**************  BOTTON BAR
    {
    bool ShowImages = false;
    Linea (); 
    BeginCenter();
    //BUTTON PREV MSG
    if (trueactualpos > 0)
      {
      truenummsg = LL->elementAt((int)(trueactualpos - 1));
      WRITE ("<A HREF=\"%s/mg_display/noop/%s/%s/%s/%d/%d/%d/%ld/\">",PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd, truenummsg);
      if (ShowImages == true) {WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_PREVMSG), L->get(ALT_PREVMSG));}
      WRITE ("[%s]\n", L->get(ALT_PREVMSG));
      WRITE ("</A>&nbsp;&nbsp;");
      }
    //BUTTON NEXT MSG
    if (trueactualpos < totalmsgs - 1)
      {
      truenummsg = LL->elementAt((int)(trueactualpos + 1));
      WRITE ("<A HREF=\"%s/mg_display/noop/%s/%s/%s/%d/%d/%d/%ld/\">",PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd, truenummsg);
      if (ShowImages == true) {WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_NEXTMSG), L->get(ALT_NEXTMSG));}
      WRITE ("[%s]\n", L->get(ALT_NEXTMSG));
      WRITE ("</A>&nbsp;&nbsp;");
      }
    if (thecclient->getServerProtocol() == PROTOCOL_IMAP) 
      {      
      //BUTTON DELNEXTMSG
      if (trueactualpos < totalmsgs - 1) {truenummsg = LL->elementAt((int)(trueactualpos + 1));} else {truenummsg = LL->elementAt((int)(trueactualpos));}
      if (isdeleted == false)
        {
        WRITE ("<A HREF=\"%s/mg_deldisplay/noop/%s/%s/%s/%d/%d/%d/%ld/\">",PATH_POSTMAN, lang, user, connid, validator, pageid, 
               actualcmd, truenummsg);
        if (ShowImages == true) {WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_DELNEXTMSG), L->get(ALT_DELNEXTMSG));}
        WRITE ("[%s]\n", L->get(ALT_DELNEXTMSG));
        WRITE ("</A>&nbsp;&nbsp;");                  
        }
      else
        {
        WRITE ("<A HREF=\"%s/mg_undeldisplay/noop/%s/%s/%s/%d/%d/%d/%ld/\">",PATH_POSTMAN, lang, user, connid, validator, pageid, 
               actualcmd, truenummsg);
        if (ShowImages == true) {WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_UNDELNEXTMSG), L->get(ALT_UNDELNEXTMSG));}
        WRITE ("[%s]\n", L->get(ALT_UNDELNEXTMSG));
        WRITE ("</A>&nbsp;&nbsp;");                          
        }
      }
    EndCenter();
    }
  } 
  
void HTML::BotoneraMailboxIndex (const char *lang, const char *user, const char *connid, UserOptions *uo)
  {
  long newbegin, actualpos, totalmsgs, aMAXMSGSFORPAGE;
  int validator, actualcmd;                           
  
  actualpos = uo->getBeginPage();
  totalmsgs = uo->getTotalMsgs();
  validator = uo->getValidator();
  actualcmd = CMD_MB_INDEX;
  
  aMAXMSGSFORPAGE = uo->getMsgsPerIndexPage();
  if (aMAXMSGSFORPAGE <= 0) aMAXMSGSFORPAGE = totalmsgs;
  
  BeginTable (0, 0, 0, 100);
    BeginTR(uo->getSkinAtribProp("COLOR_BGBOTONERA"));
      //BUTTON HELP   
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("<A HREF=\"%s/help/indexmailbox/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, 
             connid, validator, uo->getPageId(), actualcmd);
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_HELP), L->get(ALT_HELP));
      WRITE (" %s\n", L->get(ALT_HELP));
      WRITE ("</A>");
      EndTD();
      /**/BeginTD (8, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON PREVIOUS PAGE
      if (actualpos > 1)  
        {
        newbegin = actualpos - aMAXMSGSFORPAGE; if (newbegin <= 0) newbegin = 1;
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        WRITE ("<A HREF=\"%s/mb_index/noop/%s/%s/%s/%d/%d/%d/%ld/\">", PATH_POSTMAN, lang, user, connid, 
               validator, uo->getPageId(), actualcmd, newbegin);
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_PREVPAGE_ACT), L->get(ALT_PREVPAGE_ACT));
        WRITE (" %s\n", L->get(ALT_PREVPAGE_ACT));
        WRITE ("</A>");
        EndTD();
        }
      else 
        {
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_PREVPAGE_INA), L->get(ALT_PREVPAGE_INA));
        WRITE (" %s\n", L->get(ALT_PREVPAGE_INA));
        EndTD();
        }
      /**/BeginTD (8, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON NEXT PAGE
      if (actualpos + aMAXMSGSFORPAGE <= totalmsgs)
        {
        newbegin = actualpos + aMAXMSGSFORPAGE; 
        if (newbegin + aMAXMSGSFORPAGE > totalmsgs)
          {
          ldiv_t ld;
          ld = ldiv (totalmsgs, aMAXMSGSFORPAGE);
          newbegin = (ld.quot * aMAXMSGSFORPAGE) + 1;
          }
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        WRITE ("<A HREF=\"%s/mb_index/noop/%s/%s/%s/%d/%d/%d/%ld/\">", PATH_POSTMAN, lang, user, connid, validator, 
               uo->getPageId(), actualcmd, newbegin);
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_NEXTPAGE_ACT), L->get(ALT_NEXTPAGE_ACT));
        WRITE (" %s\n", L->get(ALT_NEXTPAGE_ACT));
        WRITE ("</A>");
        EndTD(); 
        }
      else
        {
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_NEXTPAGE_INA), L->get(ALT_NEXTPAGE_INA));
        WRITE (" %s\n", L->get(ALT_NEXTPAGE_INA));
        EndTD();
        }  
      /**/BeginTD (8, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON FIRST PAGE
      if (actualpos > 1)
        {
        newbegin = 1;
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        WRITE ("<A HREF=\"%s/mb_index/noop/%s/%s/%s/%d/%d/%d/%ld/\">", PATH_POSTMAN, lang, user, connid, validator, 
               uo->getPageId(), actualcmd, newbegin);
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_FIRSTPAGE_ACT), L->get(ALT_FIRSTPAGE_ACT));
        WRITE (" %s\n", L->get(ALT_FIRSTPAGE_ACT));
        WRITE ("</A>");
        EndTD();
        }          
      else
        {
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_FIRSTPAGE_INA), L->get(ALT_FIRSTPAGE_INA));
        WRITE (" %s\n", L->get(ALT_FIRSTPAGE_INA));
        EndTD();
        }  
      /**/BeginTD (8, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON LAST PAGE
      if (actualpos + aMAXMSGSFORPAGE <= totalmsgs)
        {
        ldiv_t ld;
        ld = ldiv (totalmsgs, aMAXMSGSFORPAGE);
        newbegin = (ld.quot * aMAXMSGSFORPAGE) + 1;
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        WRITE ("<A HREF=\"%s/mb_index/noop/%s/%s/%s/%d/%d/%d/%ld/\">", PATH_POSTMAN, lang, user, connid, validator, 
               uo->getPageId(), actualcmd, newbegin);
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_LASTPAGE_ACT), L->get(ALT_LASTPAGE_ACT));
        WRITE (" %s\n", L->get(ALT_LASTPAGE_ACT));
        WRITE ("</A>");
        EndTD();
        }          
      else
        {
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_LASTPAGE_INA), L->get(ALT_LASTPAGE_INA));
        WRITE (" %s\n", L->get(ALT_LASTPAGE_INA));
        EndTD();
        }
      if (thecclient->getServerProtocol() == PROTOCOL_IMAP)
        {  
        /**/BeginTD (8, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
        //BUTTON CM
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        WRITE ("<A HREF=\"%s/cm/noop/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, uo->getPageId(), actualcmd);
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_COMPOSEMSG), L->get(ALT_COMPOSEMSG));
        WRITE (" %s\n", L->get(ALT_COMPOSEMSG));
        WRITE ("</A>");
        EndTD();                
        }
      else
        {
        /**/BeginTD (8, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
        //BUTTON CMNNTP
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        WRITE ("<A HREF=\"%s/cmnntp/noop/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, uo->getPageId(), actualcmd);
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_COMPOSEMSGNNTP), L->get(ALT_COMPOSEMSGNNTP));
        WRITE (" %s\n", L->get(ALT_COMPOSEMSGNNTP));
        WRITE ("</A>");
        EndTD();                
        }
      /**/BeginTD (8, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON MB
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("<A HREF=\"%s/mb_show/noop/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, uo->getPageId(), actualcmd);
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_MAILBOXES), L->get(ALT_MAILBOXES));
      WRITE (" %s\n", L->get(ALT_MAILBOXES));
      WRITE ("</A>");
      EndTD();                      
      /**/BeginTD (8, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON OP
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("<A HREF=\"%s/op/noop/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, uo->getPageId(), actualcmd);
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_SHOWOPTIONS), L->get(ALT_SHOWOPTIONS));
      WRITE (" %s\n", L->get(ALT_SHOWOPTIONS));
      WRITE ("</A>");
      EndTD();                      
      /**/BeginTD (8, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON AB  
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("<A HREF=\"%s/ab/ab_showall/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, uo->getPageId(), actualcmd);
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_ADDBOOK), L->get(ALT_ADDBOOK));
      WRITE (" %s\n", L->get(ALT_ADDBOOK));
      WRITE ("</A>");
      EndTD();                      
      if (thecclient->getServerProtocol() == PROTOCOL_IMAP) 
        {
        /**/BeginTD (8, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
        //BUTTON MB_EXPUNGE   
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        if (uo->getConfirmPurge() == true)    
          {
          WRITE ("<A HREF=\"%s/conf_act/mb_expunge/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, uo->getPageId(), actualcmd);
          }
        else
          {  
          WRITE ("<A HREF=\"%s/mb_expunge/noop/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, uo->getPageId(), actualcmd);
          }
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_EXPUNGE), L->get(ALT_EXPUNGE));
        WRITE (" %s\n", L->get(ALT_EXPUNGE));
        WRITE ("</A>");
        EndTD();
        }        
      /**/BeginTD (8, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON LOGOUT   
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      if (uo->getConfirmLogout() == true)    
        {
        WRITE ("<A HREF=\"%s/conf_act/logout/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, uo->getPageId(), actualcmd);
        }
      else
        {  
        WRITE ("<A HREF=\"%s/logout/noop/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, uo->getPageId(), actualcmd);
        }
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_LOGOUT), L->get(ALT_LOGOUT));
      WRITE (" %s\n", L->get(ALT_LOGOUT));
      WRITE ("</A>");
      EndTD();
    EndTR();

  EndTable ();
  PrintStatusLine (user, uo, true);
  }   
  
void HTML::BotoneraComun (int cmdactual, const char *lang, const char *user, const char *connid, UserOptions *uo)
  {
  long actualpos, totalmsgs;
  int validator, pageid;
  
  actualpos = uo->getCurrentPosition();
  totalmsgs = uo->getTotalMsgs();
  validator = uo->getValidator();
  pageid    = uo->getPageId ();
  
  BeginTable (0, 0, 0, 100);
    BeginTR(uo->getSkinAtribProp("COLOR_BGBOTONERA"));
      //BUTTON BACK       
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("%s", uo->getLastStringCmd(cmdactual, lang, connid, true));
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_BACK), L->get(ALT_BACK));
      WRITE (" %s\n", L->get(ALT_BACK));
      WRITE ("</A>");
      EndTD();   
      //SPACE 
      EmptyCellx2();
      //BUTTON HELP   
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      switch (cmdactual)
        {
        case CMD_MB_SHOW: WRITE ("<A HREF=\"%s/help/mb_show/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, cmdactual); break;
        case CMD_CM: WRITE ("<A HREF=\"%s/help/cm/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, cmdactual); break;
        case CMD_OP: WRITE ("<A HREF=\"%s/help/op/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, cmdactual); break;
        case CMD_AB: case CMD_AB_SHOWALL: case CMD_AB_SORT: case CMD_AB_FROM_CM:
        case CMD_AB_ADDFIELDS:
          WRITE ("<A HREF=\"%s/help/ab/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, cmdactual); break;
        case CMD_AB_NEWENTRY:  WRITE ("<A HREF=\"%s/help/ab_newentry/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, cmdactual); break;
        case CMD_AB_EDITENTRY: WRITE ("<A HREF=\"%s/help/ab_editentry/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, cmdactual); break;
        }
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_HELP), L->get(ALT_HELP));
      WRITE (" %s\n", L->get(ALT_HELP));
      WRITE ("</A>");
      EndTD();  
      //SPACE
      EmptyCellx2();      
      //BUTTON ABOUT
      if (cmdactual == CMD_OP)
        {
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        WRITE ("<A HREF=\"%s/help/about/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, cmdactual); 
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_ABOUT), L->get(ALT_ABOUT));
        WRITE (" %s\n", L->get(ALT_ABOUT));
        WRITE ("</A>");
        EndTD();          
        EmptyCellx2();
        }
      //BUTTON LOGOUT   
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      if (uo->getConfirmLogout() == true)    
        {
        WRITE ("<A HREF=\"%s/conf_act/logout/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, cmdactual);
        }
      else
        {  
        WRITE ("<A HREF=\"%s/logout/noop/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, cmdactual);
        }
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_LOGOUT), L->get(ALT_LOGOUT));
      WRITE (" %s\n", L->get(ALT_LOGOUT));
      WRITE ("</A>");
      EndTD();
      BeginTD (100, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD(); 
    EndTR();
  EndTable ();  
  PrintStatusLine (user, uo, true);
  }

void HTML::BotoneraComposeMsg (int thecmd, const char *lang, const char *user, const char *connid, UserOptions *uo)
  {
  long actualpos, totalmsgs;
  int validator, pageid;
  
  actualpos = uo->getCurrentPosition();
  totalmsgs = uo->getTotalMsgs();
  validator = uo->getValidator();
  pageid    = uo->getPageId ();
  
  BeginTable (0, 0, 0, 100);
    BeginTR(uo->getSkinAtribProp("COLOR_BGBOTONERA"));
      //BUTTON BACK
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("%s", uo->getLastStringCmd(thecmd, lang, connid, true));      
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_BACK), L->get(ALT_BACK));
      WRITE (" %s\n", L->get(ALT_BACK));
      WRITE ("</A>");
      EndTD();                                                           
      //SPACE 
      EmptyCellx2();
      //BUTTON HELP   
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("<A HREF=\"%s/help/cm/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, thecmd); 
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_HELP), L->get(ALT_HELP));
      WRITE (" %s\n", L->get(ALT_HELP));
      WRITE ("</A>");
      EndTD();  
      //SPACE 
      EmptyCellx2();
      //BUTTON LOGOUT   
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      if (uo->getConfirmLogout() == true)    
        {
        WRITE ("<A HREF=\"%s/conf_act/logout/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, thecmd);
        }
      else
        {  
        WRITE ("<A HREF=\"%s/logout/noop/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, thecmd);
        }
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_LOGOUT), L->get(ALT_LOGOUT));
      WRITE (" %s\n", L->get(ALT_LOGOUT));
      WRITE ("</A>");
      EndTD();
      BeginTD (100, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();
    EndTR();
  EndTable ();  
  PrintStatusLine (user, uo, true);
  }

void HTML::BotoneraAttachs (const char *lang, const char *user, const char *connid, UserOptions *uo)
  {
  long actualpos, totalmsgs;
  int validator, pageid, actualcmd;
  
  actualpos = uo->getCurrentPosition();
  totalmsgs = uo->getTotalMsgs();
  validator = uo->getValidator();
  pageid    = uo->getPageId ();
  actualcmd = CMD_AT_SHOW;
  
  BeginTable (0, 0, 0, 100);
    BeginTR(uo->getSkinAtribProp("COLOR_BGBOTONERA"));
      //BUTTON BACK
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("%s", uo->getLastStringCmd(CMD_AT_SHOW, lang, connid, true));      
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_BACK), L->get(ALT_BACK));
      WRITE (" %s\n", L->get(ALT_BACK));
      WRITE ("</A>");
      EndTD();        
      //SPACE 
      EmptyCellx2();
      //BUTTON HELP   
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("<A HREF=\"%s/help/at_show/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd); 
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_HELP), L->get(ALT_HELP));
      WRITE (" %s\n", L->get(ALT_HELP));
      WRITE ("</A>");
      EndTD();  
      //SPACE 
      EmptyCellx2();
      //BUTTON LOGOUT   
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      if (uo->getConfirmLogout() == true)    
        {
        WRITE ("<A HREF=\"%s/conf_act/logout/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd);
        }
      else
        {  
        WRITE ("<A HREF=\"%s/logout/noop/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd);
        }
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_LOGOUT), L->get(ALT_LOGOUT));
      WRITE (" %s\n", L->get(ALT_LOGOUT));
      WRITE ("</A>");
      EndTD();
      BeginTD (100, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();
    EndTR();
  EndTable ();  
  PrintStatusLine (user, uo, true);
  }

void HTML::PrintIndexPage (const char *lang, const char *mailbox, const char *user, const char *connid, 
                           StringList *SLHeaders, StringList *SLMailboxes,
                           UserOptions *uo)
  {
  TBuffer pag_to_actualizar, tmpbuf;
  TSBuffer sbuf;
  TNumber anum;
  long nummsg, truenummsg, beginshow, endshow;
  uint i, cont;
  char *subject = NULL, *from = NULL;
  ldiv_t ld;
  long newbegin, aMAXMSGSFORPAGE = MAXMSGSFORPAGE;
  if (aMAXMSGSFORPAGE <= 0) aMAXMSGSFORPAGE = uo->getTotalMsgs();

  Head ("text/html", "", "");
  HTMLBegin ();
  
  ld = ldiv (uo->getTotalMsgs(), aMAXMSGSFORPAGE);
  newbegin = (ld.quot * aMAXMSGSFORPAGE) + 1;
  sprintf (pag_to_actualizar, "%s/mb_index/noop/%s/%s/%s/%d/%d/%d/%ld/", PATH_POSTMAN, lang, user, connid, uo->getValidator(), uo->getPageId(), CMD_MB_INDEX, newbegin);
  
  if (strcasecmp (mailbox, "INBOX") == 0)
    {
    Title (L->get(TIT_INDEXMAILBOX), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"), 
           uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), uo->getRefreshTime(), pag_to_actualizar);  
    }
  else
    {
    Title (L->get(TIT_INDEXMAILBOX), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"),           
           uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL);
    }
  BotoneraMailboxIndex (lang, user, connid, uo);
  nummsg = uo->getBeginPage();
  if (uo->get_use_javascript() == true)
    {
    JavaScript(DEF_JavaIndexCheck);
    }
  i = 0;
  
  BeginTable (0, 2, 1, 100);
  BeginFormPost ("PrintIndexPage1", PATH_POSTMAN);
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLEMSGINDEX"));  
  BeginTDColSpanCenter (7);
  if ((CALCULATEMAILBOXSIZE == 1) && (thecclient->getServerProtocol() == PROTOCOL_IMAP))
    {
    WRITE (L->get(L_INDEXMAILBOX3), mailbox, uo->getTotalMsgs(), FormatCountBytes1Dec(uo->getMailboxSize(), tmpbuf)); 
    }
  else
    {
    WRITE (L->get(L_INDEXMAILBOX), mailbox, uo->getTotalMsgs());
    }
  beginshow =  nummsg;  
  endshow   = nummsg - 1 + (SLHeaders->Count() / 7);
  if (endshow == 0) {beginshow = 0;}
  if (endshow > 0) {WRITE (L->get(L_MSGS_SHOWN), beginshow, endshow);}
  EndTD();
  EndTR(); 
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  if (uo->get_use_javascript() == true)
    {
    BeginTD (uo->getSkinAtribProp("COLOR_BGFILASMSGINDEX")); JSFormCheckbox ("jscheckall", "", "if (this.checked) {putOn()} else {putOff()}", false); EndTD();     
    }
  else
    {
    BeginTD(true,  1, 1, uo->getSkinAtribProp("FONT_MAIN"), 2, "center"); WRITE (HTML_HSP); EndTD(); 
    }  
  BeginTD(true,  5, 1, uo->getSkinAtribProp("FONT_MAIN"), 2, "center"); 
  if (uo->getTotalMsgs() > 0)
    {WRITE ("<A HREF=\"%s/mb_sort/number/%s/%s/%s/%d/%d/%d/\">%s</A>", PATH_POSTMAN, lang, user, connid, uo->getValidator(), uo->getPageId(), CMD_MB_INDEX, L->get(L_NUMBER));}
  else
    {WRITE ("%s", L->get(L_NUMBER));}
  EndTD();     
  BeginTD(true,  5, 1, uo->getSkinAtribProp("FONT_MAIN"), 2, "center"); 
    WRITE ("%s", L->get(L_FLAGS)); 
  EndTD();
  BeginTD(true, 10, 1, uo->getSkinAtribProp("FONT_MAIN"), 2, "center"); 
  if (uo->getTotalMsgs() > 0)
    {WRITE ("<A HREF=\"%s/mb_sort/date/%s/%s/%s/%d/%d/%d/\">%s</A>", PATH_POSTMAN, lang, user, connid, uo->getValidator(), uo->getPageId(), CMD_MB_INDEX, L->get(L_DATE));}
  else
    {WRITE ("%s", L->get(L_DATE));}
  EndTD();     
  BeginTD(true, 25, 1, uo->getSkinAtribProp("FONT_MAIN"), 2, "center"); 
  if (uo->getTotalMsgs() > 0)
    {WRITE ("<A HREF=\"%s/mb_sort/from/%s/%s/%s/%d/%d/%d/\">%s</A>", PATH_POSTMAN, lang, user, connid, uo->getValidator(), uo->getPageId(), CMD_MB_INDEX, L->get(L_FROM));}
  else
    {WRITE ("%s", L->get(L_FROM));}
  EndTD();     
  BeginTD(true, 49, 1, uo->getSkinAtribProp("FONT_MAIN"), 2, "center"); 
  if (uo->getTotalMsgs() > 0)
    {
    WRITE ("<A HREF=\"%s/mb_sort/subject/%s/%s/%s/%d/%d/%d/\">%s</A>&nbsp;&nbsp;/&nbsp;&nbsp;", PATH_POSTMAN, lang, user, connid, uo->getValidator(), uo->getPageId(), CMD_MB_INDEX, L->get(L_SUBJECT));
    WRITE ("<A HREF=\"%s/mb_sort/thread/%s/%s/%s/%d/%d/%d/\">%s</A>", PATH_POSTMAN, lang, user, connid, uo->getValidator(), uo->getPageId(), CMD_MB_INDEX, L->get(L_SORTBYTHREADS));
    }
  else
    {WRITE ("%s", L->get(L_SUBJECT));}
  EndTD();     
  BeginTD(true,  5, 1, uo->getSkinAtribProp("FONT_MAIN"), 2, "center"); 
  if (uo->getTotalMsgs() > 0)
    {WRITE ("<A HREF=\"%s/mb_sort/size/%s/%s/%s/%d/%d/%d/\">%s</A>", PATH_POSTMAN, lang, user, connid, uo->getValidator(), uo->getPageId(), CMD_MB_INDEX, L->get(L_SIZE));}
  else
    {WRITE ("%s", L->get(L_SIZE));}
  EndTD();     
  EndTR();
  
  cont = SLHeaders->Count();
  while (i < cont) 
    { 
    truenummsg = xatoldef (SLHeaders->getString(i).cstr(), 1);
    
    WRITE ("<TR valign=baseline bgcolor=\"%s\">", uo->getSkinAtribProp("COLOR_BGFILASMSGINDEX"));
    
    //CHECKBOX
    BeginTD();
    FormCheckbox ("msgnumber", xltoa(truenummsg, anum), false); EndTD();

    //1. POSITION
    WRITE ("<TD align=\"right\"><FONT size=2 face=\"%s\"><B>%s.</B></FONT></TD>\n", uo->getSkinAtribProp("FONT_TABLE"), SLHeaders->getString(i).cstr());
    ++i;
    
    if (thecclient->getServerProtocol() == PROTOCOL_IMAP)  
      {
      //2. FLAGS FOR IMAP
      WRITE ("<TD nowrap><FONT size=2 face=\"%s\"><B>%s</B></FONT></TD>\n", uo->getSkinAtribProp("FONT_TABLE"), DisplayFlags(SLHeaders->getString(i).cstr(), false, "", 0, tmpbuf));
      }
    else
      {
      //2. FLAGS FOR NNTP
      WRITE ("<TD nowrap><FONT size=2 face=\"%s\"><B>%s</B></FONT></TD>\n", uo->getSkinAtribProp("FONT_TABLE"), DisplayFlags(SLHeaders->getString(i).cstr(), false, "", 0, tmpbuf));
      }    
    
    ++i;
    //3. DATE
    WRITE ("<TD nowrap><FONT size=2 face=\"%s\"><B>%s</B></FONT></TD>\n", uo->getSkinAtribProp("FONT_TABLE"), SLHeaders->getString(i).cstr());
    ++i;

    //4. FROM
    from = Text2HTML(SLHeaders->getString((uint)i).cstr(), from);
    if (IsEmpty(from) == true) {from = xstrdup (L->get(L_NOFROM));}
    WRITE ("<TD nowrap><FONT size=2 face=\"%s\"><B>%s</B></FONT></TD>\n", uo->getSkinAtribProp("FONT_TABLE"), from);
    free (from);
    ++i;

    //5. SUBJECT
    subject = Text2HTML(SLHeaders->getString((uint)i).cstr(), subject);
    if (IsEmpty(subject) == true) {subject = xstrdup (L->get(L_NOSUBJECT));}
    WRITE ("<TD nowrap><FONT size=2 face=\"%s\"><B><A HREF=\"%s/mg_display/noop/%s/%s/%s/%d/%d/%d/%ld/\">%s</A></B></FONT></TD>\n", 
            uo->getSkinAtribProp("FONT_TABLE"), PATH_POSTMAN, lang, user, connid, uo->getValidator(), uo->getPageId(), CMD_MB_INDEX, 
            truenummsg, subject);
    free (subject);
    ++i;
    
    //6. SIZE   
    strncpy (sbuf, SLHeaders->getString(i).cstr(), CSMALLBUFFER);                                                                                                           
    if (sbuf[strlen(sbuf) - 1] != 'K')
      {
      WRITE ("<TD align=\"right\"><FONT color=\"%s\" size=2 face=\"%s\"><B>%s</B></FONT></TD>\n", RED, uo->getSkinAtribProp("FONT_TABLE"), sbuf);
      }
    else
      {
      WRITE ("<TD align=\"right\"><FONT size=2 face=\"%s\"><B>%s</B></FONT></TD>\n", uo->getSkinAtribProp("FONT_TABLE"), sbuf);
      }
    ++i;
    
    //7. UID
    //WRITE ("<TD align=\"right\"><FONT size=2 face=\"%s\"><B>%ld</B></FONT></TD>\n", uo->getSkinAtribProp("FONT_TABLE"), SLHeaders->getString(i).cstr());
    ++i;
    
    EndTR();

    ++nummsg;
    }
  EndTable();
    
  if (endshow < uo->getTotalMsgs()) //PARA PASAR PAGINA AL FINAL DEL INDEX
    {   
    newbegin = uo->getBeginPage() + aMAXMSGSFORPAGE; 
    if (newbegin + aMAXMSGSFORPAGE > uo->getTotalMsgs())
      {
      ld = ldiv (uo->getTotalMsgs(), uo->getMsgsPerIndexPage());
      newbegin = (ld.quot * uo->getMsgsPerIndexPage()) + 1;
      } 
    WRITE ("<CENTER><FONT size=2><A HREF=\"%s/mb_index/noop/%s/%s/%s/%d/%d/%d/%ld/\">[%s]</A></FONT></CENTER>", PATH_POSTMAN, lang, user, connid, uo->getValidator(),
            uo->getPageId(), CMD_MB_INDEX, newbegin,  L->get(L_NEXTPAGE_ACT));
    }
  else //ACTUALIZAR BUZON
    {
    if (thecclient->getServerProtocol() == PROTOCOL_IMAP)
      {
      //DO NOT WORK WITH NNTP. WHY?
      WRITE ("<CENTER><FONT size=2><A HREF=\"%s\">[%s]</A></FONT></CENTER>", pag_to_actualizar, L->get(L_REFRESH));
      }
    }  
  
  BeginTable (5, 0, 0, 100);
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  WRITE ("<TD><FONT>"); EndTD();
  
  FormHidden ("lang", lang);
  FormHidden ("user", user);
  FormHidden ("connid", connid);
  FormHidden ("validator", uo->getValidator());
  FormHidden ("pageid", uo->getPageId());
  FormHidden ("lastcmd", CMD_MB_INDEX);
  
  BeginTDCenter();
  FormButton ("theaction", L->get(L_MARK));
  Label (L->get(L_MSGS_AS));
  FormBeginSelect ("PrintIndexPage1", "cmd", NULL, uo->get_use_javascript()); 
  if (thecclient->getServerProtocol() == PROTOCOL_IMAP)  
    {
    FormAddOptionSelect (true,  "fg_delete",     L->get(L_DELETEMSG));
    FormAddOptionSelect (false, "fg_undelete",   L->get(L_UNDELETEMSG));
    FormAddOptionSelect (false, "fg_answered",   L->get(L_ANSWEREDMSG));
    FormAddOptionSelect (false, "fg_unanswered", L->get(L_UNANSWEREDMSG));
    FormAddOptionSelect (false, "fg_unseen",     L->get(L_UNSEENMSG));
    FormAddOptionSelect (false, "fg_seen",       L->get(L_SEENMSG));  
    FormAddOptionSelect (false, "fg_flagged",    L->get(L_FLAGGEDMSG));
    FormAddOptionSelect (false, "fg_unflagged",  L->get(L_UNFLAGGEDMSG));
    }
  else
    {
    FormAddOptionSelect (true, "fg_seen",       L->get(L_SEENMSG));          
    FormAddOptionSelect (false, "fg_unseen",     L->get(L_UNSEENMSG));
    }  
  FormEndSelect();
  EndTD();
  
  BeginTD();  WRITE (HTML_HSP); EndTD();
  
  BeginTDCenter();
  FormBeginSelect ("PrintIndexPage1", "cmd2", NULL, uo->get_use_javascript()); 
  if (thecclient->getServerProtocol() == PROTOCOL_IMAP)  
    {
    FormAddOptionSelect (false, "mg_move",     L->get(L_MOVEMSG));
    FormAddOptionSelect (false, "mg_copy",     L->get(L_COPYMSG));
    }
  FormAddOptionSelect (true,  "mb_change",   L->get(L_CHANGEMAILBOX));
  FormEndSelect();
  FormBeginSelect ("PrintIndexPage1", "mailboxname", "theaction2", uo->get_use_javascript()); 
  //FormAddOptionSelect (true, INBOX, "INBOX");
  TSBuffer abuzon;
  for (int Y=0; Y<SLMailboxes->Count(); ++Y)
    {
    //LOG ("--%s--", SLMailboxes->getString(Y).cstr());
    strncpy (abuzon, SLMailboxes->getString(Y).cstr() + strlen(HTML_DCOMILLAS), CSMALLBUFFER);
    abuzon [strlen(abuzon) - strlen(HTML_DCOMILLAS)] = '\0';
    
    XString XS;
    int ppos;
    XS = "";
    if (NCstrstr (abuzon, NNTPFOLDERHTMLPREFIX) == 1) //Mailbox from News
      {
      XS += abuzon;
      }
    else if (strcmp (abuzon, "INBOX") != 0)
      {
      ppos = NCstrstr (abuzon, uo->getMailboxPrefix());
      if (ppos > 0) {XS = XString (abuzon).substring(ppos + strlen(uo->getMailboxPrefix()) - 1);}
      else {XS = "/"; XS += abuzon;}
      }
    else {XS = "INBOX";}
    
    if (Y == 0) FormAddOptionSelect (true, SLMailboxes->getString(Y).cstr(), XS.cstr());
    else FormAddOptionSelect (false, SLMailboxes->getString(Y).cstr(), XS.cstr());
    }
  
  if (ALLOW_NNTP == 1) 
    {
    FormAddOptionSelect (false, L->get(L_ADD_MORE_FORUMS), L->get(L_ADD_MORE_FORUMS));
    }
    
  FormEndSelect(); 
  FormButton ("theaction2", L->get(L_EXECUTE));
  EndTD();
  EndTR();
  
  EndForm ();
  
  BeginFormPost ("PrintIndexPage2", PATH_POSTMAN);
  FormHidden ("lang", lang);
  FormHidden ("user", user);
  FormHidden ("connid", connid);
  FormHidden ("validator", uo->getValidator());
  FormHidden ("pageid", uo->getPageId());
  FormHidden ("cmd", "mb_search");      
  FormHidden ("lastcmd", CMD_MB_INDEX);    
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  WRITE ("<TD><FONT>"); EndTD();
  BeginTDColSpanCenter(3);   
  FormButton ("fd_tosearch", L->get(L_SEARCH));  
  FormBeginSelect ("PrintIndexPage2", "fieldtosearch", NULL, uo->get_use_javascript()); 
  FormAddOptionSelect (true,  "fd_subject", L->get(L_SEARCH_SUBJECT));
  FormAddOptionSelect (false, "fd_from",    L->get(L_SEARCH_FROM));
  FormAddOptionSelect (false, "fd_body",    L->get(L_SEARCH_BODY));
  FormAddOptionSelect (false, "fd_all",     L->get(L_SEARCH_ALL));
  FormEndSelect();
  WRITE ("<INPUT TYPE=TEXT NAME=\"whatsearch\" SIZE=\"40\" MAXLENGTH=\"100\" VALUE=\"\">\n"); 
  EndTD();
  EndTR();    
  EndForm();  
  EndTable();
  if (DEBUGGING_POSTMAN == 1) 
    {
    WRITE ("<BR><A HREF=\"%s/debug/noop/%s/%s/%s/%d/%d/%d/%ld/\">PAGINA DE DEBUG</A>\n", PATH_POSTMAN, lang, user, connid, uo->getValidator(),uo->getPageId(), CMD_MB_INDEX, uo->getCurrentPosition());
    }
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  }

void HTML::dumpAttach (const char *chunk, xulong len, StringList *SLAttachInfo)
  {
  TBuffer tiposubtipo, fn;
  strncpy (tiposubtipo, SLAttachInfo->getString(3).cstr(), CMAXBUFFER);
  strncat (tiposubtipo, "/", 1);
  strncat (tiposubtipo, SLAttachInfo->getString(4).cstr(), CMAXBUFFER);
  if (SLAttachInfo->getString(6).cstr() != NULL) strncpy (fn, SLAttachInfo->getString(6).cstr(), CMAXBUFFER);
  else if (SLAttachInfo->getString(7).cstr() != NULL) strncpy (fn, SLAttachInfo->getString(7).cstr(), CMAXBUFFER); 
  else initStr (fn);
  HeadInline (tiposubtipo, fn, len);  
  if (chunk == NULL) pcWRITE ("NULL"); 
  else pcWRITEBuffer (chunk, len);
  }  
  
void HTML::dumpRawAttachPC (const char *pcheader, xulong lenheader, const char *pcbody, xulong lenbody)
  {
  pcWRITEBuffer (pcheader, lenheader);
  pcWRITEBuffer (pcbody, lenbody);
  }    
  
void HTML::dumpRawAttachSL (StringList *SLAttachHeader, const char *pcbody, xulong lenbody)
  {
  for (int i = 0; i < SLAttachHeader->Count() - 1; ++i)
    {
    pcWRITEBuffer (SLAttachHeader->getString((uint)i).cstr(), 
                   strlen(SLAttachHeader->getString((uint)i).cstr()));
    }
  pcWRITEBuffer ("\r\n", 2);  
  pcWRITEBuffer (pcbody, lenbody);  
  }      
  
void HTML::dumpFile (TBuffer fn, TBuffer ContentType)
  {
  int fd, sz, len;
  TBuffer abuf;
  
  len = FileSize (fn);
  HeadInline (ContentType, fn, len);
  fd = open (fn, O_RDONLY);
  sz = read (fd, abuf, CMAXBUFFER);
  while (sz > 0)
    {  
    pcWRITEBuffer (abuf, sz);  
    sz = read (fd, abuf, CMAXBUFFER);
    }
  close (fd);
  }        
  
void HTML::dumpAttachWithoutHead (const char *chunk, xulong len)
  {
  if (chunk == NULL) pcWRITE ("NULL"); 
  else pcWRITEBuffer (chunk, len);
  }    
  
void HTML::PrintMessageHeader 
                        (const char *lang, const char *user, const char *connid, 
                         long nummsgs, 
                         const char *aflags, const char *ato, const char *afrom, const char *areplyto, const char *acc, const char *asubject, const char *adate, const char *asize,
                         StringList *SLAttachs,
                         UserOptions *uo, ULongList *LL)
  {
  bool isdeleted, displayreplyall;
  XString JS;
  xulong i, Partes, size;
  char *fileName, *pc = NULL;
  TBuffer attid, contenttype, basura, molde, tmpbuf;
  Head ("text/html", "", "");
  HTMLBegin ();
  Title (L->get(TIT_MESSAGEHEADER), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"),
        uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL);  
  if (strchr (aflags,  FDELETED) == NULL) isdeleted = false; else isdeleted = true;
  displayreplyall = MustDoReplayAll (ato, afrom, areplyto, acc);  
  BotoneraDisplayMsg (lang, user, connid, uo, isdeleted, LL, displayreplyall, true);
  BeginTable (0, 2, 1, 100);
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTDColSpanCenter (3); WRITE (L->get(L_MESSAGE), uo->getCurrentPosition(), nummsgs, asize); BR(); EndTD();
  EndTR();
  
  if (thecclient->getServerProtocol() == PROTOCOL_IMAP) 
    {              
    BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
    BeginTD(10); Label (L->get(L_FLAGS)); EndTD();    
    sprintf (molde, "%s/%s/%s/%d/%d/%d/%ld/\">", 
             lang, user, connid, uo->getValidator(), uo->getPageId(), CMD_MG_DISPLAY, uo->getCurrentPosition());
    BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"), "left", 1); JS = DisplayFlags(aflags, true, molde, 1, tmpbuf); WRITE (JS.cstr()); EndTD();
    BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"), "right", 1);JS = DisplayFlags(aflags, true, molde, 2, tmpbuf); WRITE (JS.cstr()); EndTD(); 
    EndTR();       
    }
  
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTD(10); Label (L->get(L_SUBJECT)); EndTD();
  BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"), "left", 2); JS = Text2HTML(asubject, pc); free (pc); WRITE (JS.cstr()); EndTD();
  EndTR();
  
  if (thecclient->getServerProtocol() == PROTOCOL_IMAP) 
    {
    BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
    BeginTD(10); Label (L->get(L_TO)); EndTD();
    BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"), "left", 2); JS = Text2HTML(ato, pc); free (pc); WRITE (JS.cstr()); EndTD();                                                 
    EndTR();
    }
  else if (thecclient->getServerProtocol() == PROTOCOL_NNTP)
    {
    BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));  
    BeginTD(10); Label (L->get(L_NNTPGROUP)); EndTD();
    BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"), "left", 2); 
    JS = Text2HTML(ato, pc); 
    free (pc); 
    WRITE (JS.cstr()); 
    EndTD();                                                 
    EndTR();    
    }
  
  //SHOW CC FIELD ONLY IF EXISTS  
  if ((acc != NULL) && (strcmp (acc, "") != 0))
    {
    BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
    BeginTD(10); Label (L->get(L_CC)); EndTD();
    BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"), "left", 2); 
    JS = Text2HTML(acc, pc); 
    free (pc); 
    WRITE (JS.cstr()); 
    EndTD();
    EndTR();  
    }
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTD(10); Label (L->get(L_FROM)); EndTD();
  BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"), "left", 2); JS = Text2HTML(afrom, pc); free (pc); WRITE (JS.cstr()); EndTD();
  EndTR();
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTD(10); Label (L->get(L_DATE)); EndTD();
  BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"), "left", 1); JS = Text2HTML(adate, pc); free (pc); WRITE (JS.cstr()); EndTD();
  BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"), "right", 1);
  sprintf (molde, "%s/mg_dumpfullheader/noop/%s/%s/%s/%d/%d/%d/%ld/", PATH_POSTMAN, lang, user, connid, uo->getValidator(), uo->getPageId(), CMD_MG_DISPLAY, uo->getCurrentPosition());
  URLIMAGE (L->get (ALT_DUMPFULLHEADER), molde, L->getI(L_DUMPFULLHEADER));
  sprintf (molde, "%s/mg_dumpfullmsg/noop/%s/%s/%s/%d/%d/%d/%ld/", PATH_POSTMAN, lang, user, connid, uo->getValidator(), uo->getPageId(), CMD_MG_DISPLAY, uo->getCurrentPosition());
  URLIMAGE (L->get (ALT_DUMPFULLMSG), molde, L->getI(L_DUMPFULLMSG));
  EndTD();
  EndTR();    
  EndTable();
  Linea ();
  
  Partes = SLAttachs->Count();
  if (Partes > 1)  //SHOW ATT INDEX WHEN THERE ARE MORE THAN 1 ATTACH
    {
    BeginFont (uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_MAIN_SIZE"), uo->getSkinAtribProp("COLOR_FONT_MAIN"));  
    H4(L->get (L_STRUCTMIME));
    for (i=0; i< Partes; ++i)
      {
      sscanf (SLAttachs->getString((xuint)i).cstr(), "%s %s %ld %s", attid, contenttype, &size, basura);
      const char *str = SLAttachs->getString(xuint(i)).cstr();
      fileName = get_filename(str);
      WRITE ("<A HREF=\"%s/at_dump/noop/%s/%s/%s/%d/%d/%d/%ld/%s/%s/\">%s</A>", 
             PATH_POSTMAN, lang, user, connid, uo->getValidator(), uo->getPageId(), CMD_MG_DISPLAY,
             uo->getCurrentPosition(), attid, fileName, SLAttachs->getString((uint)i).cstr());
      free((void *)fileName);
      if ((strstr (contenttype, "TEXT/") != NULL) && (size <= MAXTEXTSIZEATTACHFORDISPLAY))
        {
        WRITE ("<FONT COLOR=\"%s\">  %s</FONT>", RED, L->get(L_ATTACHSHOWED));
        }       
      WRITE ("<BR>\n");       
      }
    Linea ();
    EndFont();
    }
  else if (Partes == 1) //SHOW ATT INDEX WHEN THERE ARE ONLY ONE BUT THIS IS NOT TEXT/xxx
    {
    sscanf (SLAttachs->getString(0).cstr(), "%s %s %ld %s", attid, contenttype, &size, basura);
    if (strstr (contenttype, "TEXT/") == NULL)
      {
      BeginFont (uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_MAIN_SIZE"), uo->getSkinAtribProp("COLOR_FONT_MAIN"));  
      H4(L->get (L_STRUCTMIME));
      const char *str = SLAttachs->getString(0).cstr();
      fileName = get_filename(str);
      WRITE ("<A HREF=\"%s/at_dump/noop/%s/%s/%s/%d/%d/%d/%ld/%s/%s/\">%s</A>", 
             PATH_POSTMAN, lang, user, connid, uo->getValidator(), uo->getPageId(), CMD_MG_DISPLAY,
             uo->getCurrentPosition(), attid, fileName, SLAttachs->getString(0).cstr());
      free((void *)fileName);
      if ((strstr (contenttype, "TEXT/") != NULL) && (size <= MAXTEXTSIZEATTACHFORDISPLAY))
        {
        WRITE ("<FONT COLOR=\"%s\">  %s</FONT>", RED, L->get(L_ATTACHSHOWED));
        }       
      WRITE ("<BR>\n");       
      Linea ();
      EndFont();          
      }
    }  
  }  

void HTML::PrintMessageTail
                        (const char *lang, const char *user, const char *connid, 
                         long nummsgs, 
                         const char *aflags, const char *ato, const char *afrom, const char *areplyto, const char *acc, const char *asubject, const char *adate, const char *asize,
                         StringList *SLAttachs,
                         UserOptions *uo, ULongList *LL)
  {
  bool isdeleted, displayreplyall;
  if (strchr (aflags,  FDELETED) == NULL) isdeleted = false; else isdeleted = true;
  displayreplyall = MustDoReplayAll (ato, afrom, areplyto, acc);  
  BotoneraDisplayMsg (lang, user, connid, uo, isdeleted, LL, displayreplyall, false);
  }  

void HTML::PrintMailboxesPage (const char *lang, const char *mailbox, const char *user, const char *connid, 
                               StringList *SLMailboxes, UserOptions *uo)
  {
  int i;
  StringList *SLMailboxesSinComillas;
  TSBuffer abuzon;
  SLMailboxesSinComillas = new StringList();
  for (i=0; i<SLMailboxes->Count(); ++i)
    {
    strncpy (abuzon, SLMailboxes->getString(i).cstr() + strlen(HTML_DCOMILLAS), CSMALLBUFFER);
    abuzon [strlen(abuzon) - strlen(HTML_DCOMILLAS)] = '\0';
    if (strcmp(SLMailboxes->getString(i).cstr(), FORM_SELECT_SEPARATOR) == 0)
      {
      SLMailboxesSinComillas->Add (SLMailboxes->getString(i).cstr());
      }
    else {SLMailboxesSinComillas->Add (abuzon);}
    }
  
  Head ("text/html", "", "");
  HTMLBegin ();
  Title (L->get(TIT_MAILBOXES), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"),
        uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL);  
  BotoneraComun (CMD_MB_SHOW, lang, user, connid, uo);
  BeginTable (1, 2, 1, 100);
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTDColSpanCenter (2); WRITE (L->get(L_INDEXMAILBOX4), mailbox, uo->getTotalMsgs()); EndTD();
  BeginTR(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"));
  BeginFormPost ("PrintMailboxesPage1", PATH_POSTMAN);
  BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"), "right"); 
    FormButton ("theaction", L->get(L_CHANGEMAILBOX)); EndTD();   
  BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG")); 
  FormHidden ("lang", lang);
  FormHidden ("user", user);
  FormHidden ("connid", connid);
  FormHidden ("cmd", "mb_change");
  FormHidden ("validator", uo->getValidator());
  FormHidden ("pageid", uo->getPageId());
  FormHidden ("lastcmd", CMD_MB_SHOW);  
  FormBeginSelect ("PrintMailboxesPage1", "mailboxname", NULL, uo->get_use_javascript()); 
  //FormAddOptionSelect (true, INBOX, "INBOX");
  for (i=0; i<SLMailboxes->Count(); ++i)
    {
    XString XS;
    int ppos;
    XS = "";
    if (strcmp (SLMailboxesSinComillas->getString(i).cstr(), "INBOX") != 0)
      {
      ppos = NCstrstr (SLMailboxesSinComillas->getString(i).cstr(), uo->getMailboxPrefix());
      if (ppos > 0) {XS = XString (SLMailboxesSinComillas->getString(i).cstr()).substring(ppos + strlen(uo->getMailboxPrefix()) - 1);}
      else 
        {
        if (NCstrstr (SLMailboxesSinComillas->getString(i).cstr(), NNTPFOLDERHTMLPREFIX) == 1) //Mailbox from News
          {
          XS = XString (SLMailboxesSinComillas->getString(i).cstr()).substring(ppos + strlen(NNTPFOLDERPREFIX) - 1);
          }
        else {XS = "/"; XS += SLMailboxesSinComillas->getString(i).cstr();}
        }
      }
    else {XS = "INBOX";}
    if (i == 0) FormAddOptionSelect (true, SLMailboxes->getString(i).cstr(), XS.cstr()); 
    else FormAddOptionSelect (false, SLMailboxes->getString(i).cstr(), XS.cstr()); 
    }
  FormEndSelect();  
  EndTD();
  EndTR(); 
  EndForm ();
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginFormPost ("PrintMailboxesPage2", PATH_POSTMAN);
  FormHidden ("lang", lang);
  FormHidden ("user", user);
  FormHidden ("connid", connid);
  FormHidden ("cmd", "mb_create");
  FormHidden ("validator", uo->getValidator());
  FormHidden ("pageid", uo->getPageId());
  FormHidden ("lastcmd", CMD_MB_SHOW);
  BeginTD(uo->getSkinAtribProp("COLOR_BGTABLE"), "right"); FormButton ("theaction", L->get(L_CREATEMAILBOX)); EndTD();     
  BeginTD(uo->getSkinAtribProp("COLOR_BGTABLE")); FormEdit (16, 16, "mailboxname", "");     EndTD();
  EndForm ();
  EndTR();
  
  if (SLMailboxes->Count () > 0)
    {
    BeginTR(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"));
    BeginFormPost ("PrintMailboxesPage3", PATH_POSTMAN);
    FormHidden ("lang", lang);
    FormHidden ("user", user);
    FormHidden ("connid", connid);
    FormHidden ("validator", uo->getValidator());
    FormHidden ("pageid", uo->getPageId());
    FormHidden ("lastcmd", CMD_MB_SHOW);
    if (CONFIRM_DELETEMAILBOX == 1)
      {
      FormHidden ("subcmd", "mb_delete");
      FormHidden ("cmd", "conf_act");
      }
    else FormHidden ("cmd", "mb_delete");
    BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"), "right"); FormButton ("theaction", L->get(L_DELETEMAILBOX)); EndTD();
    BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"));
    FormBeginSelect ("PrintMailboxesPage3", "mailboxname", NULL, uo->get_use_javascript()); 
    for (int j=0; j<SLMailboxes->Count(); ++j)
      {
      XString XS;
      int ppos;
      XS = "";
      if (strcmp (SLMailboxesSinComillas->getString(j).cstr(), "INBOX") != 0)
        {
        ppos = NCstrstr (SLMailboxesSinComillas->getString(j).cstr(), uo->getMailboxPrefix());
        if ((ppos > 0) && (!IsEmpty(uo->getMailboxPrefix())))
          {
          XS = XString (SLMailboxesSinComillas->getString(j).cstr()).substring(ppos + strlen(uo->getMailboxPrefix()) - 1);
          }
        else 
          {
          if (NCstrstr (SLMailboxesSinComillas->getString(j).cstr(), NNTPFOLDERHTMLPREFIX) == 1) //Mailbox from News
            {
            //DO NOT ALLOW DELETE NNTP FOLDERS
            }
          else if (IsEmpty(uo->getMailboxPrefix()))
            {
            XS = XString (SLMailboxesSinComillas->getString(j).cstr());
            }  
          else 
            {
            XS = "/"; XS += SLMailboxesSinComillas->getString(j).cstr();
            }
          }
        }
      else {XS = "INBOX";}
      
      if ((XS != "INBOX") && (XS != "") 
          && (strcmp (SLMailboxes->getString(j).cstr(), FORM_SELECT_SEPARATOR) != 0))
        {       
        if (j == 0)  FormAddOptionSelect (true, SLMailboxes->getString(j).cstr(), XS.cstr());
        else FormAddOptionSelect (false, SLMailboxes->getString(j).cstr(), XS.cstr());
        }
      }
    FormEndSelect();  
    EndTD();
    EndTR();   
    EndForm ();          
    BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
    BeginFormPost ("PrintMailboxesPage4", PATH_POSTMAN);
    FormHidden ("lang", lang);
    FormHidden ("user", user);
    FormHidden ("connid", connid);
    FormHidden ("cmd", "mb_rename");
    FormHidden ("validator", uo->getValidator());
    FormHidden ("pageid", uo->getPageId());
    FormHidden ("lastcmd", CMD_MB_SHOW);
    BeginTD(uo->getSkinAtribProp("COLOR_BGTABLE"), "right"); FormButton ("theaction", L->get(L_RENAMEMAILBOX)); EndTD();
    BeginTD(uo->getSkinAtribProp("COLOR_BGTABLE"));
    FormBeginSelect ("PrintMailboxesPage4", "oldmailboxname", NULL, uo->get_use_javascript()); 
    for (int k=0; k<SLMailboxes->Count(); ++k)
      {
      XString XS;
      int ppos;
      XS = "";
      if (strcmp (SLMailboxesSinComillas->getString(k).cstr(), "INBOX") != 0)
        {
        ppos = NCstrstr (SLMailboxesSinComillas->getString(k).cstr(), uo->getMailboxPrefix());
        if ((ppos > 0) && (!IsEmpty(uo->getMailboxPrefix()))) 
          {
          XS = XString (SLMailboxesSinComillas->getString(k).cstr()).substring(ppos + strlen(uo->getMailboxPrefix()) - 1);
          }
        else 
          {
          if (NCstrstr (SLMailboxesSinComillas->getString(k).cstr(), NNTPFOLDERHTMLPREFIX) == 1) //Mailbox from News
            {
            //DO NOT ALLOW RENAME NNTP FOLDERS
            }
          else if (IsEmpty(uo->getMailboxPrefix()))
            {
            XS = XString (SLMailboxesSinComillas->getString(k).cstr());
            }              
          else 
            {
            XS = "/"; XS += SLMailboxesSinComillas->getString(k).cstr();
            }
          }
        }
      else {XS = "INBOX";}    
      
      if ((XS != "INBOX") && (XS != "") 
          && (strcmp (SLMailboxes->getString(k).cstr(), FORM_SELECT_SEPARATOR) != 0))
        {       
        if (k == 0)  FormAddOptionSelect (true, SLMailboxes->getString(k).cstr(), XS.cstr());
        else FormAddOptionSelect (false, SLMailboxes->getString(k).cstr(), XS.cstr());
        }
      }
    FormEndSelect();  
    WRITE ("%s<B>%s</B>%s\n", HTML_HSP, L->get(L_LEX_TO), HTML_HSP);
    FormEdit (16, 16, "newmailboxname", ""); 
    EndTD();
    EndTR();
    EndForm ();
    }  
  
  BeginTR(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"));
  BeginFormPost ("PrintMailboxesPage5", PATH_POSTMAN);
  FormHidden ("lang", lang);
  FormHidden ("user", user);
  FormHidden ("connid", connid);
  FormHidden ("cmd", "mb_dump");
  FormHidden ("validator", uo->getValidator());
  FormHidden ("pageid", uo->getPageId());
  FormHidden ("lastcmd", CMD_MB_SHOW);   
  BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"), "right"); FormButton ("theaction", L->get(L_DOWNLOADMAILBOX)); EndTD();     
  BeginTD(); WRITE(HTML_HSP); WRITE(HTML_HSP); Label (mailbox); EndTD();
  EndForm ();
  EndTable();          
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  delete SLMailboxesSinComillas;
  }  
        
void HTML::PrintConfirmPage (const char *stcmd, const char *stlang, const char *mailbox, 
                             const char *user, const char *connid, int validator, int pageid, UserOptions *uo)
  {
  int cmd;
  TBuffer abuff;
  cmd = TranslateCmd (stcmd);
  Head ("text/html", "", "");
  HTMLBegin ();
  Title (L->get(TIT_CONFIRMACTION), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"),
         uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL);  
  Linea ();
  BeginCenter(); 
  switch (cmd)
    {
    case CMD_MB_DELETE: {
                        sprintf (abuff, L->get(MSG_CONF_DELETEMAILBOX), mailbox); 
                        RedLabel (abuff); 
                        break;
                        }
    case CMD_LOGOUT:     RedLabel (L->get(MSG_CONF_LOGOUT)); break;
    case CMD_MB_EXPUNGE: RedLabel (L->get(MSG_CONF_EXPUNGE)); break;
    }
  EndCenter ();
  BeginCenter();
  BeginFormPost ("PrintConfirmPage1", PATH_POSTMAN);
  FormHidden ("lang", stlang);
  FormHidden ("user", user);
  FormHidden ("connid", connid);
  FormHidden ("cmd", stcmd);
  FormHidden ("validator", validator);
  FormHidden ("pageid", pageid);
  FormHidden ("lastcmd", CMD_CONF_ACT);
  if (cmd == CMD_MB_DELETE) {FormHiddenQuoted ("mailboxname", mailbox);}  
  FormButton ("theaction", L->get(L_CONFIRM));    
  EndForm ();  
  //EndTD();
  //EndTR();
  //EndTable ();
  EndCenter ();
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  }
   
void HTML::PrintComposeMsgPage (int thecmd, int subcmd, const char *lang, const char *user, const char *connid, 
                                const char *to, const char *cc, const char *bcc, const char *subject, 
                                const char *replyto, const char *from, const char *date,
                                const char *firsttext,
                                UserOptions *uo)
  {
  TSBuffer abuf;
  TBuffer amsg, temp;
  int thecmd_x, p, length_fields;
  char *pc = NULL;
  XString JS, JS2, JS3, JS4;
  Head ("text/html", "", "");
  HTMLBegin ();
  
  length_fields = uo->getWidthWriteArea() - 10;
  if ((uo->get_use_javascript() == true) && (USETIMEOUT == 1))
    {
    long restotime = (uo->getTimeout() - (5 * 60)) * 1000;
    sprintf (amsg, DEF_JavaAlertToDisabled, restotime, L->get(MSG_GO_TO_DISABLED));
    JavaScript(amsg);
    }
  
  switch (thecmd)
    {
    case CMD_CM:            Title (L->get(TIT_COMPOSEMSG), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"), uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL); break;
    case CMD_CMNNTP:        Title (L->get(TIT_COMPOSEMSGNNTP), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"), uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL); break;    
    case CMD_CM_REPLY:      Title (L->get(TIT_REPLYMSG), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"), uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL); break; 
    case CMD_CM_REPLYGROUP: Title (L->get(TIT_REPLYGROUP), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"), uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL); break;     
    case CMD_CM_REPLYALL:   Title (L->get(TIT_REPLYALLMSG), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"), uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL); break;
    case CMD_CM_FORWARD:    Title (L->get(TIT_FORWARDMSG), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"), uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL); break;
    }
  BotoneraComposeMsg (subcmd, lang, user, connid, uo);
  /*silly_table*/WRITE ("<CENTER><TABLE width=\"100%%\"><TR><TD>\n");
  BeginFormPost ("PrintComposeMsgPage1", PATH_POSTMAN);
  BeginFont (uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_MAIN_SIZE"), uo->getSkinAtribProp("COLOR_FONT_MAIN"));
  FormButton ("cm_save", L->get(L_SAVE));
  //REMOVED AT THIS MOMENT FormButton ("cm_cancel",   L->get(L_CANCELMSG));
  if ((thecclient->getServerProtocol() == PROTOCOL_IMAP) || (thecmd == CMD_CM_REPLY))
    {                       
    FormButton ("cm_send",   L->get(L_SENDMSG));
    }
  else
    {
    FormButton ("cm_sendnntp",   L->get(L_SENDMSG));
    }
  FormButton ("at_show", L->get(L_ATTACHMSG));
  FormButton ("cm_cleanall", L->get(L_CLEANALL));
  if (thecclient->getServerProtocol() == PROTOCOL_IMAP) 
    {                       
    FormButton ("ab_from_cm", L->get(L_ADDBOOK));  
    }
  FormHidden ("lang", lang);
  FormHidden ("user", user);
  FormHidden ("connid", connid);
  FormHidden ("pageid", uo->getPageId());
  FormHidden ("lastcmd", thecmd);  
  FormHidden ("cmd", "cm");
  FormHidden ("validator", uo->getValidator());
  FormHidden ("numattachs", uo->numAttachs());  
  
  //ESTO ES PARA QUE AL VOLVER A UN MENSAJE QUE ESTOY HACIENDO UN REPLY (o FORWARD) NO ME EJECUTE
  //   EL REPLY SINO QUE BUSQUE EL MSG GUARDADO Y LO MUESTRE
  thecmd_x = thecmd;
  if (uo->getLastCmd() != CMD_MG_DISPLAY) 
    {
    if (thecclient->getServerProtocol() == PROTOCOL_IMAP) 
      {
      thecmd_x = CMD_CM;
      }
    else
      {
      thecmd_x = CMD_CMNNTP;
      }  
    }
  switch (thecmd_x)
    {
    case CMD_CM: 
      BeginTable (0, 0, 1, 50);
      BeginTR(); BeginTD(); Label (L->get (L_TO));      EndTD(); BeginTD(); 
        FormEdit (MAXLENGTH_TO, length_fields, "to", Text2HTML(getLineFile(uo->getfnSavedMsg(), 2, 2, temp), pc)); free (pc);           
      EndTD(); EndTR(); 
      if (USERCANCHANGEFROM == 1)
        {
        BeginTR(); BeginTD(); Label (L->get (L_FROM)); EndTD(); BeginTD(); 
        if (strcmp (getLineFile(uo->getfnSavedMsg(), 6, 6, temp), "") == 0)
          {
          if (uo->getPersonalName() != NULL)
            {
            sprintf (temp, "%s%s%s@%s%s", uo->getPersonalName(), HTML_MENOR, user, uo->getMailDomain(), HTML_MAYOR);
            }
          else
            {
            sprintf (temp, "%s@%s", user, uo->getMailDomain());
            }
          FormEdit (MAXLENGTH_FROM, length_fields, "from", Text2HTML(temp, pc)); free (pc);
          }
        else 
          {
          FormEdit (MAXLENGTH_FROM, length_fields, "from", Text2HTML(getLineFile(uo->getfnSavedMsg(), 6, 6, temp), pc)); free (pc);    
          }       
        EndTD(); EndTR();            
        }
      else FormHidden ("from", "");  
      BeginTR(); BeginTD(); Label (L->get (L_CC));      EndTD(); BeginTD(); 
        FormEdit (MAXLENGTH_TO, length_fields, "cc", Text2HTML(getLineFile(uo->getfnSavedMsg(), 3, 3, temp), pc)); free (pc);           
      EndTD(); EndTR();
      BeginTR(); BeginTD(); Label (L->get (L_BCC));     EndTD(); BeginTD(); 
        FormEdit (MAXLENGTH_TO, length_fields, "bcc", Text2HTML(getLineFile(uo->getfnSavedMsg(), 4, 4, temp), pc)); free (pc);        
      EndTD(); EndTR();
      BeginTR(); BeginTD(); Label (L->get (L_SUBJECT)); EndTD(); BeginTD(); 
        FormEdit (255, length_fields, "subject", Text2HTML(getLineFile(uo->getfnSavedMsg(), 5, 5, temp), pc)); free (pc);
      EndTD(); EndTR();
      EndTable ();
      
      if (FileExists (uo->getfnSavedMsg()) == false)
        {
        FormCheckbox ("savemsgsentmail", bool2char(uo->getSaveMsgSentMail()), uo->getSaveMsgSentMail()); WRITE (L->get(L_SAVEMSGSENTMAIL)); WRITE ("<B>\"%s\"</B>", uo->getSentMailName()); BR();  
        }
      else
        {                                                
        FormCheckbox ("savemsgsentmail", getLineFile(uo->getfnSavedMsg(), 1, 1, temp), char2bool(getLineFile(uo->getfnSavedMsg(), 1, 1, temp))); WRITE (L->get(L_SAVEMSGSENTMAIL)); WRITE ("<B>\"%s\"</B>", uo->getSentMailName()); BR();
        }
      if (uo->numAttachs () > 0)
        {
        Label (L->get(L_ATTACHMSG));
        WRITE ("<UL>");
        for (int i = 0; i < uo->numAttachs (); ++i)
          {
          WRITE ("<LI><B>%d. %s %s</B>\n", i + 1, uo->getFileNameAttach(i), uo->getAttachInfo(i));
          }
        WRITE ("</UL>");          
        }
      Label (L->get(L_TEXTTOSEND)); BR();
      if (IsEmpty(getLineFile(uo->getfnSavedMsg(), 7, -1, temp)))
        {
        if (uo->getInsertSignatureInNewMsg() == true) 
          {JS = "\n--\n"; JS += uo->getSignature();} else {JS = "";}
        }
      else
        {
        JS = getLineFile(uo->getfnSavedMsg(), 7, -1, NULL);
        } 
      EndFont();
      FormTextArea (uo->getHeightWriteArea(), uo->getWidthWriteArea(), "texttosend", Text2HTML(JS.cstr(), pc), false); BR(); free (pc);
      break;
    case CMD_CM_REPLY:               
      BeginTable (0, 0, 1, 50);
      BeginTR(); BeginTD(); Label (L->get (L_TO));      EndTD(); BeginTD(); 
      strncpy (abuf, user, CSMALLBUFFER); strcat (abuf, "@"); strcat (abuf, uo->getMailDomain());
      //Reply to field TO or to field REPLYTO in imap mode
      //   in nntp mode we reply to "from" field ("to" field is the newsgroup)      
      if (thecclient->getServerProtocol() == PROTOCOL_NNTP)
        {
        FormEdit (MAXLENGTH_TO, length_fields, "to", Text2HTML(from, pc)); free (pc); EndTD(); EndTR();
        }
      else if ((uo->getReplyFrom() == true) || ((uo->getReplyFrom() == false) && (strcmp (replyto, "") == 0)))
        {
        //If the FROM field is the same that user@maildomain then use TO field
        if (strstr (from, abuf) == NULL)
          {
          FormEdit (MAXLENGTH_TO, length_fields, "to", Text2HTML(from, pc)); free (pc); EndTD(); EndTR();
          }
        else
          {
          FormEdit (MAXLENGTH_TO, length_fields, "to", Text2HTML(to, pc)); free (pc); EndTD(); EndTR();
          }
        }
      else
        {
        //If the REPLYTO field is the same that user@maildomain then use FROM field
        if (strstr (replyto, abuf) == NULL)
          {
          FormEdit (MAXLENGTH_TO, length_fields, "to", Text2HTML(replyto, pc)); free (pc); EndTD(); EndTR();
          }
        else
          {
          FormEdit (MAXLENGTH_TO, length_fields, "to", Text2HTML(from, pc)); free (pc); EndTD(); EndTR();
          }
        }
      if (USERCANCHANGEFROM == 1)
        {
        BeginTR(); BeginTD(); Label (L->get (L_FROM)); EndTD(); BeginTD(); 
        if (strcmp (getLineFile(uo->getfnSavedMsg(), 6, 6, temp), "") == 0)
          {
          sprintf (temp, "%s@%s", user, uo->getMailDomain());
          FormEdit (MAXLENGTH_FROM, length_fields, "from", Text2HTML(temp, pc)); free (pc);
          }
        else 
          {
          FormEdit (MAXLENGTH_FROM, length_fields, "from", Text2HTML(getLineFile(uo->getfnSavedMsg(), 6, 6, temp), pc)); free (pc);    
          }       
        EndTD(); EndTR();            
        }
      else FormHidden ("from", "");  
      BeginTR(); BeginTD(); Label (L->get (L_CC));      EndTD(); BeginTD(); FormEdit (MAXLENGTH_TO, length_fields, "cc", Text2HTML(cc, pc));   free (pc); EndTD(); EndTR();
      BeginTR(); BeginTD(); Label (L->get (L_BCC));     EndTD(); BeginTD(); FormEdit (MAXLENGTH_TO, length_fields, "bcc", Text2HTML(bcc, pc)); free (pc); EndTD(); EndTR();
      
      JS = L->get (PREFIX_REPLY);
      JS2 = XString (Text2HTML(subject, pc)).trim(); free (pc);
      p = NCstrstr (JS2.cstr(), L->get (PREFIX_REPLY));
      if (p == 1) {JS = JS2;} else {JS += (Text2HTML(subject, pc)); free (pc);}
      BeginTR(); BeginTD(); Label (L->get (L_SUBJECT)); EndTD(); BeginTD(); 
      FormEdit (255, length_fields, "subject", JS.cstr()); EndTD(); EndTR();
      EndTable ();                                                                        
      FormCheckbox ("savemsgsentmail", "1", uo->getSaveMsgSentMail()); WRITE (L->get(L_SAVEMSGSENTMAIL)); WRITE ("<B>\"%s\"</B>", uo->getSentMailName()); BR();  
      if (uo->numAttachs () > 0)
        {
        Label (L->get(L_ATTACHMSG));
        WRITE ("<UL>");
        for (int i = 0; i < uo->numAttachs (); ++i)
          {
          WRITE ("<LI><B>%d. %s %s</B>\n", i + 1, uo->getFileNameAttach(i), uo->getAttachInfo(i));
          }
        WRITE ("</UL>");          
        }      
      Label (L->get(L_TEXTTOSEND)); BR();
      JS2 = "\n"; 
      JS2 += REPLY_BEGIN_LINE;
      if (uo->getInsertSignatureInNewMsg() == true) {JS3 = "\n--\n"; JS3 += uo->getSignature(); JS3 += "\n\n";} else {JS3 = "";}      
      JS = REPLY_BEGIN_LINE; JS += firsttext;
      JS.Cut (MAXLENGTH_IN_REPLY);
      JS.replace (XString("\n"), JS2);
      JS4 = "\n\n"; JS4 += JS; JS4 += "\n\n"; JS4 += JS3;
      EndFont();
      FormTextArea (uo->getHeightWriteArea(), uo->getWidthWriteArea(), "texttosend", Text2HTML(JS4.cstr(), pc), false); free (pc); BR();
      break;
    case CMD_CMNNTP:  
      if (SHOW_NNTP_SEND_WARNING == 1)
        {
        WRITE ("<BR>\n");
        RedRectangle (L->get (MSG_NNTP_SEND_WARNING));
        WRITE ("<BR>\n");
        }
      BeginTable (0, 0, 1, 50);
      BeginTR(); BeginTD(); Label (L->get (L_TO));      EndTD(); BeginTD(); 
        Label (thecclient->getMailboxName());
      EndTD(); EndTR(); 
      FormHidden ("to", thecclient->getMailboxName());

      if (USERCANCHANGEFROM == 1)
        {
        BeginTR(); BeginTD(); Label (L->get (L_FROM)); EndTD(); BeginTD(); 
        if (strcmp (getLineFile(uo->getfnSavedMsg(), 6, 6, temp), "") == 0)
          {
          if (uo->getPersonalName() != NULL)
            {
            sprintf (temp, "%s%s%s@%s%s", uo->getPersonalName(), HTML_MENOR, user, uo->getMailDomain(), HTML_MAYOR);
            }
          else
            {
            sprintf (temp, "%s@%s", user, uo->getMailDomain());
            }
          FormEdit (MAXLENGTH_FROM, length_fields, "from", Text2HTML(temp, pc)); free (pc);
          }
        else 
          {
          FormEdit (MAXLENGTH_FROM, length_fields, "from", Text2HTML(getLineFile(uo->getfnSavedMsg(), 6, 6, temp), pc)); free (pc);   
          }       
        EndTD(); EndTR();            
        }
      else FormHidden ("from", "");  
      BeginTR(); BeginTD(); Label (L->get (L_SUBJECT)); EndTD(); BeginTD(); 
        FormEdit (255, length_fields, "subject", Text2HTML(getLineFile(uo->getfnSavedMsg(), 5, 5, temp), pc)); free (pc);
      EndTD(); EndTR();
      EndTable ();
      
      if (FileExists (uo->getfnSavedMsg()) == false)
        {
        FormCheckbox ("savemsgsentmail", bool2char(uo->getSaveMsgSentMail()), uo->getSaveMsgSentMail()); WRITE (L->get(L_SAVEMSGSENTMAIL)); WRITE ("<B>\"%s\"</B>", uo->getSentMailName()); BR();  
        }
      else
        {                                                
        FormCheckbox ("savemsgsentmail", getLineFile(uo->getfnSavedMsg(), 1, 1, temp), char2bool(getLineFile(uo->getfnSavedMsg(), 1, 1, temp))); WRITE (L->get(L_SAVEMSGSENTMAIL)); WRITE ("<B>\"%s\"</B>", uo->getSentMailName()); BR();
        }
      if (uo->numAttachs () > 0)
        {
        Label (L->get(L_ATTACHMSG));
        WRITE ("<UL>");
        for (int i = 0; i < uo->numAttachs (); ++i)
          {
          WRITE ("<LI><B>%d. %s %s</B>\n", i + 1, uo->getFileNameAttach(i), uo->getAttachInfo(i));
          }
        WRITE ("</UL>");          
        }
      Label (L->get(L_TEXTTOSEND)); BR();
      if (IsEmpty(getLineFile(uo->getfnSavedMsg(), 7, -1, temp)))
        {
        if (uo->getInsertSignatureInNewMsg() == true) 
          {JS = "\n--\n"; JS += uo->getSignature();} else {JS = "";}
        }
      else
        {
        JS = getLineFile(uo->getfnSavedMsg(), 7, -1, NULL);
        } 
      EndFont();
      FormTextArea (uo->getHeightWriteArea(), uo->getWidthWriteArea(), "texttosend", Text2HTML(JS.cstr(), pc), false); free (pc); BR();
      break;
    case CMD_CM_REPLYGROUP:               
      if (SHOW_NNTP_SEND_WARNING == 1)
        {
        WRITE ("<BR>\n");
        RedRectangle (L->get (MSG_NNTP_SEND_WARNING));
        WRITE ("<BR>\n");
        }
      BeginTable (0, 0, 1, 50);
      BeginTR(); BeginTD(); Label (L->get (L_TO));      EndTD(); BeginTD(); 
      strncpy (abuf, user, CSMALLBUFFER); strcat (abuf, "@"); strcat (abuf, uo->getMailDomain());
      
      //Reply to News Group      
      Label (thecclient->getMailboxName());
      FormHidden ("to", thecclient->getMailboxName());
      
      if (USERCANCHANGEFROM == 1)
        {
        BeginTR(); BeginTD(); Label (L->get (L_FROM)); EndTD(); BeginTD(); 
        if (strcmp (getLineFile(uo->getfnSavedMsg(), 6, 6, temp), "") == 0)
          {
          sprintf (temp, "%s@%s", user, uo->getMailDomain());
          FormEdit (MAXLENGTH_FROM, length_fields, "from", Text2HTML(temp, pc)); free (pc);
          }
        else 
          {
          FormEdit (MAXLENGTH_FROM, length_fields, "from", Text2HTML(getLineFile(uo->getfnSavedMsg(), 6, 6, temp), pc)); free (pc);
          }       
        EndTD(); EndTR();            
        }
      else FormHidden ("from", "");  
      
      JS = L->get (PREFIX_REPLY);
      JS2 = XString (Text2HTML(subject, pc)).trim(); free (pc);
      p = NCstrstr (JS2.cstr(), L->get (PREFIX_REPLY));
      if (p == 1) {JS = JS2;} else {JS += (Text2HTML(subject, pc)); free(pc);}
      BeginTR(); BeginTD(); Label (L->get (L_SUBJECT)); EndTD(); BeginTD(); 
      FormEdit (255, length_fields, "subject", JS.cstr()); EndTD(); EndTR();
      EndTable ();                                                                        
      FormCheckbox ("savemsgsentmail", "1", uo->getSaveMsgSentMail()); WRITE (L->get(L_SAVEMSGSENTMAIL)); WRITE ("<B>\"%s\"</B>", uo->getSentMailName()); BR();  
      if (uo->numAttachs () > 0)
        {
        Label (L->get(L_ATTACHMSG));
        WRITE ("<UL>");
        for (int i = 0; i < uo->numAttachs (); ++i)
          {
          WRITE ("<LI><B>%d. %s %s</B>\n", i + 1, uo->getFileNameAttach(i), uo->getAttachInfo(i));
          }
        WRITE ("</UL>");          
        }      
      Label (L->get(L_TEXTTOSEND)); BR();
      JS2 = "\n"; 
      JS2 += REPLY_BEGIN_LINE;
      if (uo->getInsertSignatureInNewMsg() == true) {JS3 = "\n--\n"; JS3 += uo->getSignature(); JS3 += "\n\n";} else {JS3 = "";}      
      JS = REPLY_BEGIN_LINE; JS += firsttext;
      JS.Cut (MAXLENGTH_IN_REPLY);
      JS.replace (XString("\n"), JS2);
      JS4 = "\n\n"; JS4 += JS; JS4 += "\n\n"; JS4 += JS3;
      EndFont();
      FormTextArea (uo->getHeightWriteArea(), uo->getWidthWriteArea(), "texttosend", Text2HTML(JS4.cstr(), pc), false); free(pc); BR();
      break;      
    case CMD_CM_REPLYALL:               
      BeginTable (0, 0, 1, 50);
      BeginTR(); BeginTD(); Label (L->get (L_TO));      EndTD(); BeginTD(); 
      strncpy (abuf, user, CSMALLBUFFER); strcat (abuf, "@"); strcat (abuf, uo->getMailDomain());
      
      FormEdit (MAXLENGTH_TO, length_fields, "to", Text2HTML(getReplyAllAddresses(to, from, replyto, cc), pc)); free (pc); EndTD(); EndTR();        
      if (USERCANCHANGEFROM == 1)
        {
        BeginTR(); BeginTD(); Label (L->get (L_FROM)); EndTD(); BeginTD(); 
        if (strcmp (getLineFile(uo->getfnSavedMsg(), 6, 6, temp), "") == 0)
          {
          sprintf (temp, "%s@%s", user, uo->getMailDomain());
          FormEdit (MAXLENGTH_FROM, length_fields, "from", Text2HTML(temp, pc)); free (pc);
          }
        else 
          {
          FormEdit (MAXLENGTH_FROM, length_fields, "from", Text2HTML(getLineFile(uo->getfnSavedMsg(), 6, 6, temp), pc)); free(pc);    
          }       
        EndTD(); EndTR();            
        }
      else FormHidden ("from", "");  
      BeginTR(); BeginTD(); Label (L->get (L_CC));      EndTD(); BeginTD(); FormEdit (MAXLENGTH_TO, length_fields, "cc", "");           EndTD(); EndTR();
      BeginTR(); BeginTD(); Label (L->get (L_BCC));     EndTD(); BeginTD(); FormEdit (MAXLENGTH_TO, length_fields, "bcc", "");         EndTD(); EndTR();
      
      JS = L->get (PREFIX_REPLY);
      JS2 = XString (Text2HTML(subject, pc)).trim(); free(pc); 
      p = NCstrstr (JS2.cstr(), L->get (PREFIX_REPLY));
      if (p == 1) {JS = JS2;} else {JS += (Text2HTML(subject, pc)); free (pc);}
      BeginTR(); BeginTD(); Label (L->get (L_SUBJECT)); EndTD(); BeginTD(); 
      FormEdit (255, length_fields, "subject", JS.cstr()); EndTD(); EndTR();
      EndTable ();                                                                        
      FormCheckbox ("savemsgsentmail", "1", uo->getSaveMsgSentMail()); WRITE (L->get(L_SAVEMSGSENTMAIL)); WRITE ("<B>\"%s\"</B>", uo->getSentMailName()); BR();  
      if (uo->numAttachs () > 0)
        {
        Label (L->get(L_ATTACHMSG));
        WRITE ("<UL>");
        for (int i = 0; i < uo->numAttachs (); ++i)
          {
          WRITE ("<LI><B>%d. %s %s</B>\n", i + 1, uo->getFileNameAttach(i), uo->getAttachInfo(i));
          }
        WRITE ("</UL>");          
        }      
      Label (L->get(L_TEXTTOSEND)); BR();
      JS2 = "\n"; 
      JS2 += REPLY_BEGIN_LINE;
      if (uo->getInsertSignatureInNewMsg() == true) {JS3 = "\n--\n"; JS3 += uo->getSignature(); JS3 += "\n\n";} else {JS3 = "";}      
      JS = REPLY_BEGIN_LINE; JS += firsttext;
      JS.Cut (MAXLENGTH_IN_REPLY);
      JS.replace (XString("\n"), JS2);
      JS4 = "\n\n"; JS4 += JS; JS4 += "\n\n"; JS4 += JS3;
      EndFont();
      FormTextArea (uo->getHeightWriteArea(), uo->getWidthWriteArea(), "texttosend", Text2HTML(JS4.cstr(), pc), false); free (pc); BR();
      break;            
    case CMD_CM_FORWARD:
      BeginTable (0, 0, 1, 50);
      BeginTR(); BeginTD(); Label (L->get (L_TO));      EndTD(); BeginTD(); 
      FormEdit (MAXLENGTH_TO, length_fields, "to", ""); EndTD(); EndTR();
      if (USERCANCHANGEFROM == 1)
        {
        BeginTR(); BeginTD(); Label (L->get (L_FROM)); EndTD(); BeginTD(); 
        if (strcmp (getLineFile(uo->getfnSavedMsg(), 6, 6, temp), "") == 0)
          {
          sprintf (temp, "%s@%s", user, uo->getMailDomain());
          FormEdit (MAXLENGTH_FROM, length_fields, "from", Text2HTML(temp, pc)); free (pc);
          }
        else 
          {
          FormEdit (MAXLENGTH_FROM, length_fields, "from", Text2HTML(getLineFile(uo->getfnSavedMsg(), 6, 6, temp), pc)); free (pc);    
          }       
        EndTD(); EndTR();            
        }
      else FormHidden ("from", "");  
      BeginTR(); BeginTD(); Label (L->get (L_CC));      EndTD(); BeginTD(); FormEdit (MAXLENGTH_TO, length_fields, "cc", Text2HTML(cc, pc));   free (pc); EndTD(); EndTR();
      BeginTR(); BeginTD(); Label (L->get (L_BCC));     EndTD(); BeginTD(); FormEdit (MAXLENGTH_TO, length_fields, "bcc", Text2HTML(bcc, pc)); free (pc); EndTD(); EndTR();
      JS = L->get (PREFIX_FORWARD); JS += (Text2HTML(subject, pc)); free (pc);
      BeginTR(); BeginTD(); Label (L->get (L_SUBJECT)); EndTD(); BeginTD(); 
      FormEdit (255, length_fields, "subject", JS.cstr()); EndTD(); EndTR();
      EndTable ();
      FormCheckbox ("savemsgsentmail", "1", uo->getSaveMsgSentMail()); WRITE (L->get(L_SAVEMSGSENTMAIL)); WRITE ("<B>\"%s\"</B>", uo->getSentMailName()); BR();  
      if (uo->numAttachs () > 0)
        {
        Label (L->get(L_ATTACHMSG));
        WRITE ("<UL>");
        for (int i = 0; i < uo->numAttachs (); ++i)
          {
          WRITE ("<LI><B>%d. %s %s</B>\n", i + 1, uo->getFileNameAttach(i), uo->getAttachInfo(i));
          }
        WRITE ("</UL>");          
        }
      Label (L->get(L_TEXTTOSEND)); BR();
      JS = "\n\n\n";
      if (uo->getInsertSignatureInNewMsg() == true) {JS += "--\n"; JS += uo->getSignature(); JS += "\n\n";}
      JS += L->get(FORWARDLINE); JS += "\n"; 
      if (strcmp(to, "")     != 0) {JS += L->get(L_TO);      JS += to;      JS += "\n";}
      if (strcmp(subject, "")!= 0) {JS += L->get(L_SUBJECT); JS += subject; JS += "\n";}
      if (strcmp(replyto, "")!= 0) {JS += L->get(L_REPLYTO); JS += replyto; JS += "\n";}
      if (strcmp(from, "")   != 0) {JS += L->get(L_FROM);    JS += from;    JS += "\n";}
      if (strcmp(date, "")   != 0) {JS += L->get(L_DATE);    JS += date;    JS += "\n";}
      JS += "\n"; 
      JS += firsttext;  
      EndFont();
      FormTextArea (uo->getHeightWriteArea(), uo->getWidthWriteArea(), "texttosend", Text2HTML(JS.cstr(), pc), false); free (pc); BR();      
      break;  
    }
  EndForm();   
  /*end_silly_table*/WRITE("</TD></TR></TABLE></CENTER>\n");  
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  }

void HTML::PrintShowOptions (const char *stlang, const char *user, const char *connid, UserOptions *uo)
  {
  TNumber anum;
  bool iscurrent;
  Head ("text/html", "", "");
  HTMLBegin ();
  Title (L->get(TIT_SHOWOPTIONS), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"),
        uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL);
  BotoneraComun (CMD_OP, stlang, user, connid, uo);
  BeginFont (uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_MAIN_SIZE"), uo->getSkinAtribProp("COLOR_FONT_MAIN"));
  BeginFormPost ("PrintShowOptions1", PATH_POSTMAN);
  FormHidden ("lang", stlang);
  FormHidden ("user", user);
  FormHidden ("connid", connid);
  FormHidden ("pageid", uo->getPageId());
  FormHidden ("lastcmd", CMD_OP);
  FormHidden ("cmd", "op");
  FormHidden ("validator", uo->getValidator());
  Label (L->get(L_SHOWOPTIONS)); BR();
  
  BeginTable (5, 0, 0, 25);
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTD(); BeginCenter(); FormButton ("op_cancel", L->get(L_CANCEL)); EndCenter(); EndTD();
  BeginTD(); BeginCenter(); FormButton ("op_save",   L->get(L_SAVE));   EndCenter(); EndTD();
  EndTR(); 
  EndTable();
  
  Linea();
  Label (L->get(L_PERSONALNAME)); FormEdit (200, 50, "personalname", uo->getPersonalName()); BR();  
  FormCheckbox ("savemsgsentmail", "1", uo->getSaveMsgSentMail()); WRITE (L->get(L_SAVEMSGSENTMAIL)); FormEdit (50, 20, "sentmailname", uo->getSentMailName()); BR(); 
  FormCheckbox ("replyfrom", "1", uo->getReplyFrom()); WRITE (L->get(L_REPLYFROM)); BR(); 
  FormCheckbox ("forwattachs", "1", uo->getForwAttachs()); WRITE (L->get(L_FORWATTACHS)); BR(); 
  Linea();
  Label (L->get(L_MSGSPERINDEXPAGE)); FormEdit (5, 5, "msgsperindexpage", xltoa(uo->getMsgsPerIndexPage(), anum)); BR();
  Label (L->get(L_TRUNCATELENGTHREADINGMSG)); 
  FormEdit (5, 5, "truncatelengthreadingmsg", xltoa(uo->getTruncateLengthReadingMsg(), anum)); BR();
  Label (L->get(L_SIZEWRITEAREA)); 
  FormEdit (5, 5, "heightwritearea", xltoa(uo->getHeightWriteArea(), anum)); Label (" / ");
  FormEdit (5, 5, "widthwritearea", xltoa(uo->getWidthWriteArea(), anum)); BR();
  Label (L->get(L_REFRESHTIME)); 
  FormEdit (5, 5, "refreshtime", xltoa(uo->getRefreshTime(), anum)); BR();
  FormCheckbox ("confirmpurge",  "1", uo->getConfirmPurge());  WRITE (L->get(L_CONFIRMPURGE));  BR();
  FormCheckbox ("confirmlogout", "1", uo->getConfirmLogout()); WRITE (L->get(L_CONFIRMLOGOUT)); BR();
  FormCheckbox ("showclock", "1", uo->getShowClock()); WRITE (L->get(L_OP_SHOWCLOCK)); BR();  
  Label (L->get(L_SKIN));
  FormBeginSelect ("PrintShowOptions1", "skin", NULL, uo->get_use_javascript());
  if (strcmp (uo->getSkin(), "DEFAULT") == 0) {iscurrent = true;} else {iscurrent = false;} FormAddOptionSelect (iscurrent,  "DEFAULT", "DEFAULT");
  if (strcmp (uo->getSkin(), "OLIVE")   == 0) {iscurrent = true;} else {iscurrent = false;} FormAddOptionSelect (iscurrent, "OLIVE",   "OLIVE");
  if (strcmp (uo->getSkin(), "RED")     == 0) {iscurrent = true;} else {iscurrent = false;} FormAddOptionSelect (iscurrent, "RED",   "RED");
  if (strcmp (uo->getSkin(), "BLUE")    == 0) {iscurrent = true;} else {iscurrent = false;} FormAddOptionSelect (iscurrent, "BLUE",   "BLUE");  
  FormEndSelect();
  BR();
  Linea();
  FormCheckbox ("insigninnewmsg", "1", uo->getInsertSignatureInNewMsg()); WRITE (L->get(L_INSIGNINNEWMSG)); BR();    
  Label (L->get(L_SIGNATURE)); BR(); 
  EndFont();
  FormTextArea (10, 55, "textsignature", uo->getSignature(), false); BR();  
  
  if (ALLOW_NNTP == 1) //News Groups
    {        
    TBuffer cutgroup;
    int pk, ppos;

    Linea ();
    Label (L->get(L_NNTPGROUP)); BR();
    FormButton ("op_nntp", L->get(L_MANAGE_NNTPGROUPS)); BR();  
    Label (L->get(L_SUBSCRIBEDGROUPS)); BR();
    
    BeginTable (1, 0, 0, 100);
    ppos = 0;
    while (ppos < uo->SLSubscribedGroups->Count())
      {
      pk = 0;
      BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
      while (pk < 3)
        {
        BeginTD();
        if (ppos < uo->SLSubscribedGroups->Count())
          {  
          WRITE (CutSign (uo->SLSubscribedGroups->getString(ppos).cstr(), 35, CUTINDICATOR, cutgroup));
          }
        else
          {
          WRITE ("&nbsp;");
          }  
        EndTD();
        ++ppos;        
        ++pk;
        }     
      EndTR();  
      }
    EndTable(); 
    }
    
  BR();
  Linea();
  BeginTable (5, 0, 0, 25);
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTD(); BeginCenter(); FormButton ("op_cancel", L->get(L_CANCEL)); EndCenter(); EndTD();
  BeginTD(); BeginCenter(); FormButton ("op_save",   L->get(L_SAVE));   EndCenter(); EndTD();
  EndTR(); 
  EndTable();

  EndForm ();
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  }
     
void HTML::PrintInvalidPage (const char *stlang, const char *user, const char *connid, UserOptions *uo)
  {
  Head ("text/html", "", "");
  HTMLBegin ();
  Title (L->get(TIT_INVALIDPAGE), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"),
         uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL);
  RedLabel (L->get(MSG_INVALIDPAGE)); BR();
  WRITE ("<A HREF=\"%s/mb_index/noop/%s/%s/%s/%d/%d/%d/%ld/\">", PATH_POSTMAN, stlang, user, connid, uo->getValidator(), 
         uo->getPageId(), uo->getLastCmd(), uo->getCurrentPosition());
  WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_BACK), L->get(L_BACK));
  WRITE ("</A>");
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  }
          
void HTML::PrintBadCookiePage (const char *stlang, const char *user, const char *connid, UserOptions *uo)
  {
  Head ("text/html", "", "");
  HTMLBegin ();
  Title (L->get(TIT_BADCOOKIE), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"),
         uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL);
  WRITE ("<CENTER><img src=\"%s\" border=0 alt=\"%s\"></CENTER>", L->getI(L_SECURITYPROBLEM), L->get(L_SECURITYPROBLEM));
  WRITE ("<CENTER>\n");
  RedLabel (L->get(MSG_BADCOOKIE)); BR();
  WRITE ("</CENTER>\n");
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  }                    
          
void HTML::PrintShowAttachs (const char *stlang, const char *user, const char *connid, UserOptions *uo)
  {
  TNumber anum;
  char *pc = NULL;
  XString XS;
  Head ("text/html", "", "");
  HTMLBegin ();
  Title (L->get(TIT_SHOWATTACHS), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"),
        uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL);
  BotoneraAttachs (stlang, user, connid, uo);
  Linea ();
  BeginFont (uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_MAIN_SIZE"), uo->getSkinAtribProp("COLOR_FONT_MAIN"));    
  if (uo->numAttachs() > 0)
    {
    BeginFormPost ("PrintShowAttachs1", PATH_POSTMAN);
    FormHidden ("lang", stlang);
    FormHidden ("user", user);
    FormHidden ("connid", connid);
    FormHidden ("cmd", "cm");
    FormHidden ("validator", uo->getValidator());    
    FormHidden ("pageid", uo->getPageId());
    FormHidden ("lastcmd", CMD_AT_SHOW);    
    Label (L->get(L_ATTACHMSG)); BR();    
    for (int i=0; i < uo->numAttachs(); ++i) 
      {
      FormCheckbox ("attnumber", xltoa(i, anum), false); 
      XS = ""; XS += xitoa(i + 1, anum); XS += ". "; XS += Text2HTML(uo->getFileNameAttach(i), pc); free (pc); XS += " "; 
      XS += Text2HTML(uo->getAttachInfo(i), pc); free (pc); XS += "<BR>\n";
      WRITE ("<B><FONT COLOR=\"%s\">%s</FONT></B>", RED, XS.cstr());
      }
    BR();
    FormButton ("at_erase", L->get(L_DETACH));
    EndForm ();
    }   
  else
    {
    Label (L->get(L_NO_ATTACHS)); BR();
    }
  Linea ();
  BeginFormPostMultipartFormData ("PrintShowAttachs2", PATH_POSTMAN);
  FormHidden ("lang", stlang);
  FormHidden ("user", user);
  FormHidden ("connid", connid);
  FormHidden ("cmd", "cm");
  FormHidden ("validator", uo->getValidator());  
  FormHidden ("pageid", uo->getPageId());
  FormHidden ("lastcmd", CMD_AT_SHOW);
  Label (L->get(L_ATTACH_NEW)); BR();
  Label (L->get(L_FILE)); FormFile (255, 30, "file", "", "*"); BR();
  FormButton ("at_save", L->get(L_DOATTACH));
  EndForm ();
  EndFont();
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  }
  
void HTML::PrintAddBook_Show (int thecmd, const char *stlang, const char *user, const char *connid, 
                              UserOptions *uo, Addressbook *ab, int sortby)
  {
  TBuffer abuf;
  TNumber anum;
  Head ("text/html", "", "");
  HTMLBegin ();
  Title (L->get(TIT_ADDBOOK), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"),
         uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL);
  BotoneraComun (thecmd, stlang, user, connid, uo);
  
  BeginTable (1, 2, 1, 100);
  
  //CASILLA NUMBER OF ENTRIES
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTDColSpanCenter (10); WRITE ("<B>%s: %d</B>\n", L->get(L_NUMOFENTRIES), ab->Count ()); EndTD();
  EndTR();
  
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  //BOTON NEW ENTRY
  BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
  WRITE ("<A HREF=\"%s/ab/ab_newentry/%s/%s/%s/%d/%d/%d/\">\n", PATH_POSTMAN, stlang, user, connid, 
          uo->getValidator(), uo->getPageId(), thecmd);
  WRITE ("<img src=\"%s\" border=0 alt=\"%s\">\n", L->getI(L_AB_NEWENTRY_S), L->get(ALT_AB_NEWENTRY_S));
  WRITE (" %s\n", L->get(L_AB_NEWENTRY_S));
  WRITE ("</A>\n");
  EndTD();
  //BOTON DUMP          
  BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
  WRITE ("<A HREF=\"%s/ab/ab_dump/%s/%s/%s/%d/%d/%d/\">\n", PATH_POSTMAN, stlang, user, connid, 
         uo->getValidator(), uo->getPageId(), thecmd); 
  WRITE ("<img src=\"%s\" border=0 alt=\"%s\">\n", L->getI(L_AB_DUMP_S), L->get(ALT_AB_DUMP_S));
  WRITE (" %s\n", L->get(L_AB_DUMP_S));
  WRITE ("</A>\n");
  EndTD(); 
  //BOTON ADD SELECTED ADDRESSES  
  BeginTDColSpanCenter (8);        
  BeginBold();
  BeginFormPost ("PrintAddBook_Show1", PATH_POSTMAN);
  FormHidden ("lang", stlang);
  FormHidden ("user", user);
  FormHidden ("connid", connid);
  FormHidden ("validator", uo->getValidator());
  FormHidden ("pageid", uo->getPageId());
  FormHidden ("lastcmd", uo->getLastCmd());  
  FormHidden ("cmd", "ab_addfields");
  FormButton ("theaction", L->get(L_ADDADDRESS_FIELD));  
  EndBold();
  EndTD();
  EndTR();
  
  //LABELS
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTD(); Label("&nbsp;"); EndTD();
  BeginTD(); Label(L->get(L_TO0)); EndTD();
  BeginTD(); Label(L->get(L_CC0)); EndTD();
  BeginTD(); Label(L->get(L_BCC0)); EndTD();
  BeginTD(); Label("&nbsp;"); EndTD();

  sprintf (abuf, "%s/ab/ab_sort/%s/%s/%s/%d/%d/%d/0/", PATH_POSTMAN, stlang, user, connid, 
           uo->getValidator(), uo->getPageId(), uo->getLastCmd()); 
  BeginTD(); URL (L->get(L_AB_NICKNAME), abuf); EndTD();
  
  sprintf (abuf, "%s/ab/ab_sort/%s/%s/%s/%d/%d/%d/1/", PATH_POSTMAN, stlang, user, connid, 
           uo->getValidator(), uo->getPageId(), uo->getLastCmd()); 
  BeginTD(); URL (L->get(L_AB_FULLNAME), abuf); EndTD();
    
  sprintf (abuf, "%s/ab/ab_sort/%s/%s/%s/%d/%d/%d/2/", PATH_POSTMAN, stlang, user, connid, 
           uo->getValidator(), uo->getPageId(), uo->getLastCmd()); 
  BeginTD(); URL (L->get(L_AB_ADDRESSES), abuf); EndTD();   

  sprintf (abuf, "%s/ab/ab_sort/%s/%s/%s/%d/%d/%d/3/", PATH_POSTMAN, stlang, user, connid, 
           uo->getValidator(), uo->getPageId(), uo->getLastCmd()); 
  BeginTD(); URL (L->get(L_ADDBOOK), abuf); EndTD();     
  
  BeginTD(); WRITE ("<U>%s</U>\n", L->get(L_AB_COMMENTS)); EndTD();
  EndTR();
  ab->SORT(sortby);

  WRITE ("<FONT size=2 face=\"%s\" color=\"%s\">\n", TABLEFACEFONT, BLACK);

  for (int N = 0; N < ab->Count (); ++N)
    {    
    if ((IsMultiplo (N, 20) == true) && (N > 0))
      {
      BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));

      WRITE ("<TD><B>&nbsp;</B></TD>\n");
      WRITE ("<TD><B>%s</B></TD>\n", L->get(L_TO0));
      WRITE ("<TD><B>%s</B></TD>\n", L->get(L_CC0));            
      WRITE ("<TD><B>%s</B></TD>\n", L->get(L_BCC0));
      WRITE ("<TD><B>&nbsp;</B></TD>\n");      
      WRITE ("<TD><B>%s</B></TD>\n", L->get(L_AB_NICKNAME));      
      WRITE ("<TD><B>%s</B></TD>\n", L->get(L_AB_FULLNAME));
      WRITE ("<TD><B>%s</B></TD>\n", L->get(L_AB_ADDRESSES));
      WRITE ("<TD><B>%s</B></TD>\n", L->get(L_ADDBOOK));
      WRITE ("<TD><B>%s</B></TD>\n", L->get(L_AB_COMMENTS));            
      EndTR();      
      }     
    WRITE ("<TR valign=baseline bgcolor=\"%s\">\n", uo->getSkinAtribProp("COLOR_BGFILASADDBOOK"));
    WRITE ("<TD><B>%s</B></TD>\n", xitoa(N + 1, anum));

    WRITE ("<TD>"); FormCheckbox ("numentryto",  xitoa(N + 1, anum), false); WRITE ("</TD>\n");
    WRITE ("<TD>"); FormCheckbox ("numentrycc",  xitoa(N + 1, anum), false); WRITE ("</TD>\n");
    WRITE ("<TD>"); FormCheckbox ("numentrybcc", xitoa(N + 1, anum), false); WRITE ("</TD>\n");
    WRITE ("<TD>"); 
      sprintf (abuf, "%s/ab/ab_editentry/%s/%s/%s/%d/%d/%d/%d/", 
               PATH_POSTMAN, stlang, user, connid, uo->getValidator(), uo->getPageId(), uo->getLastCmd(), N); 
      URL (L->get(L_EDIT), abuf);
    WRITE ("</TD>\n");
    WRITE ("<TD>"); WRITE (CutSign(ForceSpace(ab->getNickname  (N)), 20, CUTINDICATOR, abuf)); WRITE ("</TD>\n");
    WRITE ("<TD>"); WRITE (CutSign(ForceSpace(ab->getFullname  (N)), 20, CUTINDICATOR, abuf)); WRITE ("</TD>\n");
    WRITE ("<TD>"); WRITE (CutSign(ForceSpace(ab->getAddresses (N)), 20, CUTINDICATOR, abuf)); WRITE ("</TD>\n");
    WRITE ("<TD>"); WRITE (CutSign(ForceSpace(ab->getFcc       (N)), 20, CUTINDICATOR, abuf)); WRITE ("</TD>\n");
    WRITE ("<TD>"); WRITE (CutSign(ForceSpace(ab->getComment   (N)), 20, CUTINDICATOR, abuf)); WRITE ("</TD>\n");
    
    EndTR();
    }
  WRITE ("</FONT>\n");
  EndTable ();  
  
  EndForm ();
    
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  }         

void HTML::PrintAddBook_Dump (int thecmd, const char *stlang, const char *user, const char *connid, 
                              UserOptions *uo, Addressbook *ab, int sortby)
  {
  int NUM;
  XString as;
  Head ("text/html", "", "");
  ab->SORT(sortby);
  NUM = ab->Count ();
  if (NUM <= 0)
    {
    WRITE ("%s\n", L->get(MSG_NO_ELEMENTS));
    }
  else
    {  
    WRITE ("<PRE>\n");
    for (int N = 0; N < ab->Count (); ++N)
      {
      WRITE ("<BR>#%d ------------------------------------------\n", N + 1);
      WRITE ("<BR>%s: %s\n", L->get(L_AB_NICKNAME),  ab->getNickname  (N));    
      WRITE ("<BR>%s: %s\n", L->get(L_AB_FULLNAME),  ab->getFullname  (N));
      WRITE ("<BR>%s: %s\n", L->get(L_ADDBOOK),   ab->getFcc       (N));         
      WRITE ("<BR>%s: %s\n", L->get(L_AB_COMMENTS),  ab->getComment   (N));
      as = ab->getAddresses (N); 
      as.replace (",", "<BR>");
      WRITE ("<BR>%s\n", as.cstr());    
      }
    WRITE ("</PRE>\n");      
    }
  }

void HTML::PrintAddBook_EditEntry (int thecmd, const char *stlang, const char *user, const char *connid, 
                                   UserOptions *uo, Addressbook *ab, int numentry)
  {
  TNumber anum;
  char *pc = NULL;
  Head ("text/html", "", "");
  HTMLBegin ();
  Title (L->get(TIT_ADDBOOK), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"),
         uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL);
  BotoneraComun (thecmd, stlang, user, connid, uo);
  Linea();
  BeginFont (uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_MAIN_SIZE"), uo->getSkinAtribProp("COLOR_FONT_MAIN"));  
  BeginFormPost ("PrintAddBook_EditEntry1", PATH_POSTMAN);
  FormHidden ("lang", stlang);
  FormHidden ("user", user);
  FormHidden ("connid", connid);
  FormHidden ("cmd", "ab");
  FormHidden ("numentry", xitoa(numentry, anum));
  FormHidden ("validator", uo->getValidator()); 
  FormHidden ("pageid", uo->getPageId());
  FormHidden ("lastcmd", thecmd);    
  FormButton ("ab_saveentry", L->get(L_SAVE));
  if (thecmd == CMD_AB_EDITENTRY) FormButton ("ab_deleentries", L->get(L_DELETE));
  else FormButton ("ab_deleentries", L->get(L_CANCEL));
  BeginTable (1, 2, 1, 100);
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTD(); Label (L->get(L_AB_NICKNAME)); EndTD();
  BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG")); 
  if (numentry > -1) {FormEdit (255, 50, "nickname", Text2HTML(ab->getNickname(numentry), pc)); free (pc);}
  else {FormEdit (255, 50, "nickname", "");}
  EndTD();  
  EndTR();
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTD(); Label (L->get(L_AB_FULLNAME)); EndTD();
  BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG")); 
  if (numentry > -1) {FormEdit (255, 50, "fullname", Text2HTML(ab->getFullname(numentry), pc)); free (pc);}
  else {FormEdit (255, 50, "fullname", "");}
  EndTD();  
  EndTR();
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTD(); Label (L->get(L_AB_ADDRESSES)); EndTD();                
  BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG")); 
  if (numentry > -1) {FormEdit (MAXLENGTH_TO, 50, "addresses", Text2HTML(ab->getAddresses(numentry), pc)); free (pc);} 
  else {FormEdit (MAXLENGTH_TO, 50, "addresses", "");}
  EndTD();  
  EndTR();  
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTD(); Label (L->get(L_ADDBOOK)); EndTD();                
  BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG")); 
  if (numentry > -1) {FormEdit (255, 50, "fcc", Text2HTML(ab->getFcc(numentry), pc)); free (pc);} 
  else {FormEdit (255, 50, "fcc", "");}
  EndTD();  
  EndTR();    
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTD(); Label (L->get(L_AB_COMMENTS)); EndTD();
  BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG")); 
  if (numentry > -1) {FormEdit (255, 50, "comments", Text2HTML(ab->getComment(numentry), pc)); free (pc);} 
  else {FormEdit (255, 50, "comments", "");}
  EndTD();  
  EndTR();  
  EndTable ();  
  EndForm ();
  EndFont();

  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  }           
    
void HTML::PrintDebugPage (const char *stlang, const char *user, const char *connid, UserOptions *uo)
  {
  Head ("text/html", "", "");
  HTMLBegin ();
  WRITE ("DEBUG\n");
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  }

void HTML::PrintFullHeaders (StringList *SLHeaders, UserOptions *uo, const char *asize)
  {
  TBuffer abuf;
  XString JSUno, JSDos;
  int p;
  char *pc = NULL;
    
  Head ("text/html", "", "");
  HTMLBegin ();
  Title (L->get(TIT_SHOWHEADERS), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"),
        uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL);  
  BeginTable (0, 2, 1, 100);
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTDColSpanCenter (3); BeginBold(); WRITE (L->get(TIT_SHOWHEADERS)); EndBold(); EndTD(); EndTR();
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTDColSpanCenter (3); WRITE (L->get(L_MESSAGE), uo->getCurrentPosition(), uo->getTotalMsgs(), asize); EndTD(); EndTR();
  for (int i=0; i< SLHeaders->Count(); ++i)
    {
    BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));    
    strncpy (abuf, SLHeaders->getString(i).cstr(), CMAXBUFFER);
    p = NCstrstr (abuf, ":");
    if ((p > 0) && (abuf[0] != SP) && (abuf[0] != '\x009'))
      {
      JSUno = XString (abuf).substring (0, p);
      JSDos = XString (abuf).substring (p); 
      }
    else
      {
      JSUno = "";
      JSDos = abuf;
      }
    JSDos.replace ("\n", "");   
    BeginTD(10); WRITE ("<B>%s</B>", Text2HTML(JSUno.cstr(), pc)); free (pc); EndTD();
    BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"), "left", 2);
    WRITE (Text2HTML(JSDos.cstr(), pc));  free (pc);
    EndTD();
    EndTR();
    }
  EndTable();  
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  }

void HTML::PrintTCPTimeoutPage (const char *lang)
  {        
  Head ("text/html", "", "");
  HTMLBegin ();
  Title (L->get(L_ERROR), WHITE, BLACK, DEFIMG_BG, false, 0, NULL);
  WRITE ("<p><h1><FONT color=\"%s\">%s:</font> %s</h1><p>\n",  RED, L->get(L_ERROR), L->get(ERR_SERVERISDOWN));
  WRITE ("<p><A HREF=\"%s?lang=%s\">%s</A>", PATH_POSTMAN, lang, L->get(MSG_CANRECONNECTHERE));
  WRITE ("<HR noshade size=2>\n");
  WRITE ("<SMALL>%s</SMALL>\n", L->get (L_COPYRIGHT));
  WRITE ("</BODY>\n");
  WRITE ("</HTML>\n");    
  }

void HTML::PrintShowNNTPManageGroups (const char *stlang, const char *user, const char *connid, UserOptions *uo, 
                                      ClientNNTP *cn, const char *text2search) 
  {
  TBuffer abuf;
  
  Head ("text/html", "", "");
  HTMLBegin ();
  Title (L->get(TIT_SHOWNNTPMANAGEGROUPS), uo->getSkinAtribProp ("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"),
        uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL);
  BotoneraComun (CMD_OP_NNTP, stlang, user, connid, uo);
  BeginFont (uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_MAIN_SIZE"), uo->getSkinAtribProp("COLOR_FONT_MAIN"));

  BeginFormPost ("PrintShowNNTPManageGroups1", PATH_POSTMAN);
  FormHidden ("lang", stlang);
  FormHidden ("user", user);
  FormHidden ("connid", connid);
  FormHidden ("pageid", uo->getPageId());
  FormHidden ("lastcmd", CMD_OP);
  FormHidden ("cmd", "op_nntp");
  FormHidden ("validator", uo->getValidator());
  Label (L->get(L_SHOWOPTIONS)); BR();
  
  BeginTable (5, 0, 0, 25);
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTD(); BeginCenter(); FormButton ("op_nntp_cancel", L->get(L_CANCEL)); EndCenter(); EndTD();
  BeginTD(); BeginCenter(); FormButton ("op_nntp_save",   L->get(L_SAVE));   EndCenter(); EndTD();
  EndTR(); 
  EndTable();

  BR();
  Linea();
  
  Label (L->get(L_NNTPSERVER)); Label (uo->getNNTPServer()); BR();
  
  BR();
  Linea();

  Label (L->get(L_SUBSCRIBEDGROUPS)); BR();
  FormTextArea (10, 75, "subscribedgroups", uo->SLSubscribedGroups->toPChar(true), false); BR();

  if (uo->getObligatedGroups(abuf) > 0)
    {
    Small ("("); Small (L->get(L_FORCEDGROUPS)); Small (abuf); Small (")");
    }
    
  Linea();
  Label (L->get(L_GROUPSTOADD)); BR();
  FormButton ("op_nntp_search", L->get(L_NNTP_LISTSEARCH));
  FormEdit (255, 30, "text2search", text2search ? text2search : BEGIN_FILTER_NNTP_SEARCH); 
  sprintf (abuf, "%s/nw_list_dump/noop/%s/%s/%s/%d/%d/%d/", PATH_POSTMAN, stlang, user, connid, 
           uo->getValidator(), uo->getPageId(), uo->getLastCmd()); 
  BeginTD(); URL (L->get(L_DUMPFULLLIST), abuf); EndTD();
  BR();
    
  if (cn->SLSearchedGroups->Count() > 0)
    {
    FormBeginMultipleSelect ("PrintShowNNTPManageGroups1", "searchedgroups", NULL, uo->get_use_javascript()); 
    for (int i = 0; i < cn->SLSearchedGroups->Count(); ++i)
      {
      FormAddOptionSelect (false, cn->SLSearchedGroups->getString(i).cstr(), cn->SLSearchedGroups->getString(i).cstr());
      }
    FormEndSelect();
    BR();
    FormButton ("op_nntp_addselected", L->get(L_ADDSELECTEDGROUPS));
    FormButton ("op_nntp_open", L->get(L_GOSELECTEDGROUP));
    BR();
    }
  else
    {
    FormBeginMultipleSelect ("PrintShowNNTPManageGroups1", "searchedgroups", NULL, uo->get_use_javascript()); 
    FormAddOptionSelect (false, L->get(L_LIST_EMPTY), L->get(L_LIST_EMPTY));
    FormEndSelect();
    BR ();
    }  
    
  Linea();
  BeginTable (5, 0, 0, 25);
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTD(); BeginCenter(); FormButton ("op_nntp_cancel", L->get(L_CANCEL)); EndCenter(); EndTD();
  BeginTD(); BeginCenter(); FormButton ("op_nntp_save",   L->get(L_SAVE));   EndCenter(); EndTD();
  EndTR(); 
  EndTable();

  EndForm ();
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  }
     
          
